#include "clerk.h"
using namespace std;

const port_t clerk::arrive = 0;
const port_t clerk::depart = 1;

void clerk::empty_line()
{
	while (!line.empty())
	{
		delete line.front();
		line.pop_front();
	}
}

void clerk::init()
{
	empty_line();
	passivate();
}

void clerk::delta_ext(double e, const adevs_bag<PortValue>& x)
{
	/// Add the new customers to the back of the line.
	for (int i = 0; i < x.getSize(); i++)
	{
		/// Cast the object* values to a customer* type.
		const customer* new_customer = dynamic_cast<const customer*>(x.get(i).value);
		/// Copy the customer object and place it at the back of the line.
		line.push_back(new customer(*new_customer));
		/// Record the time at which the customer entered the line.
		line.back()->tenter = timeCurrent();
	}
	/// If we are not already processing a customer,
	if (timeNext() == ADEVS_INFINITY)
	{
		/// Then schedule our next event when the customer should depart.
		hold(line.front()->twait);
		/// Record the time at which the customer will leave the line.
		line.front()->tleave = timeNext() - line.front()->twait;
	}
	/// Otherwise, continue with our next event time unchanged.

	cout << "Computed the external transition function at t = " << timeCurrent() << endl;
	cout << "The last event time was " << timeLast() << endl;
	cout << "There are " << line.size() << " customers waiting." << endl;
	cout << "The next customer will leave at t = " << timeNext() << "." << endl;
}
	
void clerk::delta_int()
{
	/// Remove the departing customer from the front of the line.
	/// The departing customer will be deleted later by our garbage
	/// collection method.
	line.pop_front();
	/// Check to see if any customers are waiting.
	if (line.empty())
	{
		/// If not, then set our time of next event to infinity.
		passivate();
	}
	else
	{
		/// Otherwise, start ringing up the next customer in line.
		/// Schedule our next event when the customer should depart.
		hold(line.front()->twait);
		/// Record the time at which the customer will leave the line.
		line.front()->tleave = timeNext() - line.front()->twait;
	}

	cout << "Computed the internal transition function at t = " << timeCurrent() << endl;
	cout << "The last event time was " << timeLast() << endl;
	cout << "There are " << line.size() << " customers waiting." << endl;
	cout << "The next customer will leave at t = " << timeNext() << "." << endl;
}

void clerk::delta_conf(const adevs_bag<PortValue>& x)
{
	delta_int();
	delta_ext(0.0,x);
}

void clerk::output_func(adevs_bag<PortValue>& y)
{
	/// Place the customer at the front of the line onto the depart port.
	/// A corresponding PortValue object is added to the bag y by the output()
	/// method.
	output(depart,line.front(),y);
	
	cout << "Computed the output function at t = " << timeCurrent() << endl;
	cout << "The last event time was " << timeLast() << endl;
	cout << "A customer just departed!" << endl;
}

void clerk::gc_output(adevs_bag<PortValue>& g)
{
	for (int i = 0; i < g.getSize(); i++)
	{
		delete g[i].value;
	}
}

clerk::~clerk()
{
	empty_line();
}
