#include "clerk2.h"
#include <iostream>
using namespace std;

const port_t clerk2::arrive = 0;
const port_t clerk2::depart = 1;
const double clerk2::SMALL_ORDER = 1.0;
const double clerk2::PREEMPT_TIME = 5.0;

void clerk2::empty_line()
{
	while (!line.empty())
	{
		delete line.front();
		line.pop_front();
	}
}

void clerk2::init()
{
	preempt = 0.0;
	empty_line();
	passivate();
}

void clerk2::delta_ext(double e, const adevs_bag<PortValue>& x)
{
	/// Decrement the waiting time for the current customer
	if (!line.empty())
	{
		line.front()->twait -= e;
	}
	/// Reduce the preempt time
	preempt -= e;
	/// Place new customers into the line
	for (int i = 0; i < x.getSize(); i++)
	{
		cout << "A new customer arrived at t = " << timeCurrent() << endl;
		/// Cast the object* values to a customer* type.
		customer* new_customer = 
			new customer(*(dynamic_cast<const customer*>(x.get(i).value)));
		/// Record the time at which the customer enters the line
		new_customer->tenter = timeCurrent();
		/// If the customer has a small order
		if (preempt <= 0.0 && new_customer->twait <= SMALL_ORDER)
		{
			cout << "The new customer has preempted the current one!" << endl;
			/// We won't preempt another customer for at least this long
			preempt = PREEMPT_TIME;
			/// Put the new customer at the front of the line
			line.push_front(new_customer);
		}
		/// otherwise just put the customer at the end of the line
		else 
		{
			cout << "The new customer is at the back of the line" << endl;
			line.push_back(new_customer);
		}
	}
	/// Process the first customer in the line
	line.front()->tleave = timeCurrent();
	hold(line.front()->twait);
}
	
void clerk2::delta_int()
{
	/// Update the preemption timer
	preempt -= ta();
	/// Remove the departing customer from the front of the line.
	/// The departing customer will be deleted later by our garbage
	/// collection method.
	line.pop_front();
	/// Check to see if any customers are waiting.
	if (line.empty())
	{
		cout << "The line is empty at t = " << timeCurrent() << endl;
		/// If not, then set our time of next event to infinity.
		passivate();
		return;
	}
	/// If the preemption time has passed, then look for a small
	/// order that can be promoted to the front of the line.
	list<customer*>::iterator i;
	for (i = line.begin(); i != line.end() && preempt <= 0.0; i++)
	{
		if ((*i)->twait <= SMALL_ORDER)
		{
			cout << "A queued customer has a small order at time " << 
			timeCurrent() << endl;
			customer* small_order = *i;
			line.erase(i);
			line.push_front(small_order);
			preempt = PREEMPT_TIME;
			break;
		}
	}
	/// Otherwise, start ringing up the first customer in line.
	/// Schedule our next event when the customer should depart.
	hold(line.front()->twait);
	/// Record the time at which the customer left the line.
	line.front()->tleave = timeCurrent();
}

void clerk2::delta_conf(const adevs_bag<PortValue>& x)
{
	delta_int();
	delta_ext(0.0,x);
}

void clerk2::output_func(adevs_bag<PortValue>& y)
{
	/// Place the customer at the front of the line onto the depart port.
	/// A corresponding PortValue object is added to the bag y by the output()
	/// method.
	output(depart,line.front(),y);

	cout << "A customer departed at t = " << timeNext() << endl;
}

void clerk2::gc_output(adevs_bag<PortValue>& g)
{
	for (int i = 0; i < g.getSize(); i++)
	{
		delete g[i].value;
	}
}

clerk2::~clerk2()
{
	empty_line();
}
