#include "decision.h"
#include <iostream>
using namespace std;

// Assign identifiers to ports.  Assumes NUM_LINES = 2.
// The numbers are selected to allow indexing into the
// line length and port number arrays.
const port_t decision::departures[NUM_LINES] = { 0, 1 };
const port_t decision::arrive[NUM_LINES] = { 0, 1 };
// Inport port for arriving customer that need to make a decision
const port_t decision::decide = NUM_LINES;

decision::decision():
atomic()
{
}

void decision::init()
{
	// Clear the list of deciders that have joined their lines
	clear_deciders();
	// Set the initial line lengths to zero
	for (int i = 0; i < NUM_LINES; i++)
	{
		line_length[i] = 0;
	}
	// Wait for a customer that needs to decide
	passivate();
}

void decision::delta_int()
{
	// Move out all of the deciders
	deciding.clear();
	// Wait for a new customer
	passivate();
}

void decision::delta_ext(stime_t e, const adevs_bag<PortValue>& x)
{
	// Assign new arrivals to a line and update the line length
	port_iterator new_customers(x,decide);
	for (; !new_customers.end_of_input(); new_customers++)
	{
		pair<int,object*> p(find_shortest_line(),(*new_customers)->clone());
		deciding.push_back(p);
		line_length[p.first]++;
	}
	// Decrement the length of lines that had customers leave
	for (int i = 0; i < NUM_LINES; i++)
	{
		port_iterator old_customers(x,i);
		for (; !old_customers.end_of_input(); old_customers++)
		{
			line_length[i]--;
		}
	}
	// If there are customers getting into line, then produce output
	// immediately.
	if (!deciding.empty())
	{
		hold(0.0);
	}
	// Otherwise, wait for another customer
	else
	{
		passivate();
	}
}
		
void decision::delta_conf(const adevs_bag<PortValue>& x)
{
	delta_int();
	delta_ext(0.0,x);
}

void decision::output_func(adevs_bag<PortValue>& y)
{
	// Send all customers to their lines
	list<pair<port_t,object*> >::iterator i = deciding.begin();
	for (; i != deciding.end(); i++)
	{
		output((*i).first,(*i).second,y);
	}
}

void decision::gc_output(adevs_bag<PortValue>& g)
{
	for (int i = 0; i < g.getSize(); i++)
	{
		delete g[i].value;
	}
}

decision::~decision()
{
	clear_deciders();
}

void decision::clear_deciders()
{
	list<pair<port_t,object*> >::iterator i = deciding.begin();
	for (; i != deciding.end(); i++)
	{
		delete (*i).second;
	}
	deciding.clear();
}

port_t decision::find_shortest_line()
{
	port_t shortest = 0;
	for (port_t i = 0; i < NUM_LINES; i++)
	{
		if (line_length[shortest] > line_length[i])
		{
			shortest = i;
		}
	}
	return shortest;
}
