#ifndef __decision_h_
#define __decision_h_
#include "adevs.h"
#include <list>

// Number of lines to consider.
#define NUM_LINES 2

class decision: public atomic 
{
	public:
		/// Constructor.
		decision();
		/// State initialization function.
		void init();
		/// Internal transition function.
		void delta_int();
		/// External transition function.
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		/// Confluent transition function.
		void delta_conf(const adevs_bag<PortValue>& x);
		/// Output function.  
		void output_func(adevs_bag<PortValue>& y);
		/// Output value garbage collection.
		void gc_output(adevs_bag<PortValue>& g);
		/// Destructor.
		~decision();
		/// Input port that receives new customers
		static const port_t decide;
		/// Input ports that receive customers leaving the two lines
		static const port_t departures[NUM_LINES];
		/// Output ports that produce customers for the two lines
		static const port_t arrive[NUM_LINES];

	private:
		/// Lengths of the two lines
		int line_length[NUM_LINES];
		/// List of deciding customers and their decision.
		std::list<std::pair<port_t,object*> > deciding;
		/// Delete all waiting customers and clear the list.
		void clear_deciders();
		/// Returns the arrive port associated with the shortest line
		port_t find_shortest_line();
};

#endif
