#ifndef __cell_h_
#define __cell_h_
#include "adevs.h"
#include <fstream>

/// Indexes into initial condition data
#define LEFT 0
#define RIGHT 1
#define UP 2
#define DOWN 3
#define CENTER 4

/**
A cell used in the finite difference solution of the 2D heat equation.
Each cell corresponds to a grid point in the spatial approximation.
*/
class cell: public atomic, public cell_interface
{
	public:
		/// Quantum size
		static double D;
		/// Grid resolution in the x and y dimensions
		static double dx, dy;
		/// Diffusion rate
		static double k;
		/// Dimensions of the cellspace
		static int width, height;
		/**
		If true, use adiabatic boundary conditions.
		Use fixed (= 0) boundary conditions if false.
		*/
		static bool adiabatic;
 		
		cell();
		/// Initialization function
		void init();
		/// Internal transition function
		void delta_int();
		/// External transition function
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		/// Confluent transition function
		void delta_conf(const adevs_bag<PortValue>& x);
		/// Output function
		void output_func(adevs_bag<PortValue>& _y);
		void gc_output(adevs_bag<PortValue>& g);
		void report(std::ofstream& stream) const;
		~cell();
		void set_init_conditions(double u0[5]);
		void set_pos(int x, int y);
	private:
		/// Quantized integrator
		qode q;
		/// Value of differential function at this point
		double du;
		/// Position in the cell space
		int x, y;
		/// Compute derivative at cell and handle initial conditions
		void compute_derivative(double* delta_u);
};

#endif
