#include "configuration.h"
#include <iostream>
using namespace std;

configuration::configuration(string config_file):
adiabatic(false),
dx(0.0),
dy(0.0),
D(0.0),
tend(0.0),
k(0.0),
granule(0.0),
width(0),
height(0)
{
	data_buf = NULL;
	string field;
	ifstream data(config_file.c_str());
	/// Load simulation parameters from a file
	while (!data.eof())
	{
		data >> field;
		/// Get the discretization of space in the x direction
		if (field == "dx") data >> dx;
		/// Get the time granule ('close enough' to zero) value used by the sim. engine
		else if (field == "granule") data >> granule;
		/// Get the discretization in the y direction
		else if (field == "dy") data >> dy;
		/// Get the number of cells in the x direction
		else if (field == "width") data >> width;
		/// Get the number of cells in the y direction
		else if (field == "height") data >> height;
		/// Get the quantum size used by the integrator
		else if (field == "D") data >> D;
		/// Get the diffusion constant for the cellspace
		else if (field == "k") data >> k;
		/// Get the file that holds the initial conditions data
		else if (field == "udata") data >> u_data;
		/// Get the simulation stop time
		else if (field == "tend") data >> tend;
		/// Get the boundary conditions
		else if (field == "adiabatic") adiabatic = true;
		/// Ignore comments
		else if (field == "#")
		{
			do
			{
				data >> field;
			}
			while (field != "#");
		}
	}
	data.close();
	/// Load the initial condition data
	load_data();
}

void configuration::load_data()
{
	done();
	ifstream data(u_data.c_str());
	/// Allocate space to hold initial conditions
	data_buf = new double*[get_width()];
	for (int i = 0; i < get_width(); i++)
	{
		data_buf[i] = new double[get_height()];
	}
	/// Load initial conditions from file
	for (int i = 0; i < get_width(); i++)
	{
		for (int j = 0; j < get_height(); j++)
		{
			data >> data_buf[i][j];
		}
	}
	data.close();
}

void configuration::done()
{
	/// Free up memory used to hold initial conditions
	if (data_buf == NULL) return;
	for (int i = 0; i < get_width(); i++)
	{
		delete [] data_buf[i];
	}
	delete [] data_buf;
	data_buf = NULL;
}

configuration::~configuration()
{
	done();
}
