#ifndef __configuration__h_
#define __configuration__h_
#include <string>
#include <fstream>
#include <math.h>
/**
Load and provide access to initialization data.
*/
class configuration
{
	public:
		/// Load and parse a configuration file
		configuration(std::string config_file);
		/// Get choice of boundary conditions
		bool get_adiabatic() const { return adiabatic; }
		/// Get the width of the cell space
		int get_width() const { return width; }
		/// Get the height of the cell space
		int get_height() const { return height; }
		/// Get the time granule to be used by the simulation engine
		double get_granule() const { return granule; }
		/// Get the simulation end time
		double get_tend() const { return tend; }
		/// Get the width of each cell (i.e. discretization in the x direction)
		double get_dx() const { return dx; }
		/// Get the height of each cell (i.e. discretization in the y direction)
		double get_dy() const { return dy; }
		/// Get the quantum size to be used to integrate at each cell
		double get_D() const { return D; }
		/// Get the diffusion constant
		double get_k() const { return k; }
		/// Get the initial conditions stored in a 2D array
		double** get_data() { return data_buf; }
		/// Free the memory used to hold the initial conditions
		void done();
		~configuration();
	private:
		bool adiabatic;
		double dx, dy, D, tend, k, granule;
		double** data_buf;
		int width, height;
		std::string u_data;

		void load_data();
};

#endif
