#include "configuration.h"
#include "adevs.h"
#include "cell.h"
#include <iostream>
#include <fstream>
#include <stdio.h>
using namespace std;

int main(int argc, char** argv)
{
	if (argc < 2)
	{
		std::cerr << argv[0] << " <configuration file>" << std::endl;
		std::cerr << "No input file" << std::endl;
		return 0;
	}
	/// Load the configuration file specified on the command line
	configuration config(argv[1]);
	/// Setup the cell parameters
	cell::width = config.get_width();
	cell::height = config.get_height();
	cell::D = config.get_D();
	cell::dx = config.get_dx();
	cell::dy = config.get_dy();
	cell::k = config.get_k();
	cell::adiabatic = config.get_adiabatic();
	/// Create a 2D cellspace.  In the 2D cell space each cell communicates
	/// with its four neighbors immediately above, below, left, and right.
	/// In 3D there would have been six neighbors, hence the SIX_POINT flag
	/// passed to the cellspace constructor.
	cellSpace space(cellSpace::SIX_POINT,config.get_width(),config.get_height());
	/// Set the initial conditions at each cell and add the cells to the space
	for (int x = 0; x < config.get_width(); x++)
	{
		for (int y = 0; y < config.get_height(); y++)
		{
			cell* c = new cell;
			c->set_pos(x,y);
			double uu[5];
			for (int i = 0; i < 5; i++) uu[i] = config.get_data()[x][y];
			if (x > 0) uu[LEFT] = config.get_data()[x-1][y];
			if (x < config.get_width()-1) uu[RIGHT] = config.get_data()[x+1][y];
			if (y > 0) uu[DOWN] = config.get_data()[x][y-1];
			if (y < config.get_height()-1) uu[UP] = config.get_data()[x][y+1];
			c->set_init_conditions(uu);
			space.add(c,x,y);
		}
	}
	/// Done with setup, free initial data
	config.done();

	/// Run the simulation
	try
	{
		devssim sim(&space);	
		sim.run(config.get_tend());
	}
	catch(adevs_exception& err)
	{
		cout << err << endl;
		throw;
	}
	/// Dump the solution at tend to a file
	ofstream log_file("soln");
	for (int x = 0; x < config.get_width(); x++)
	{
		for (int y = 0; y < config.get_height(); y++)
		{
			const cell* c = dynamic_cast<const cell*>(space.getModel(x,y));
			c->report(log_file);
		}
		log_file << endl;
	}
	log_file.close();
	/// Done!
	return 0;
}
