#ifndef __configuration__h_
#define __configuration__h_
#include <string>
#include <fstream>
#include <math.h>
#include "fireCellWind.h"

/**
Load and provide access to initialization data.
*/
class configuration
{
	public:
		// Load the specified configuration data file
		configuration(std::string config_file);
		// Get the width of the cellspace
		int get_width() const { return width; }
		// Get the height of the cellspace
		int get_height() const { return height; }
		// Get the simulation end time
		int get_tend() const { return tend; }
		// Get the amount of fuel at location x,y
		int get_fuel(int x, int y) const { return fuel[x][y]; }
		// True indicates that x,y is initial burning
		bool get_fire(int x, int y) const { return fire[x][y]; }
		// Get the wind direction
		Direction get_wind() const { return wind; }
		// Destructor
		~configuration();
	private:
		
		void load_wind();
		void load_fire();
		void load_fuel();
		std::ifstream data;
		int tend, width, height;
		int** fuel;
		bool** fire;
		Direction wind;
};

#endif
