#include "fireCell.h"
using namespace std;

fireCell::fireCell():
atomic(),
cell_interface()
{
}

void fireCell::set_init_conditions(double f, bool burning)
{
	heat = 0.0;
	fuel = f;
	if (burning)
	{
		phase = BURNING;
	}
	else
	{
		phase = UNBURNT;
	}
}

void fireCell::init()
{
	if (phase == BURNING)
	{
		hold(1.0);
	}
	else
	{
		passivate();
	}
}

void fireCell::delta_int()
{
	fuel = fuel - elapsed();
	// If the phase is burnt, then the cell is burned up.
	if (phase == BURNT)
	{
		passivate();
	}
	// Otherwise, hold for as long as the cell has fuel and
	// then produce an output indicating that the cell is burnt up.
	else
	{
		hold(max(fuel,0.0));
		phase = BURNT;
	}
}

void fireCell::delta_ext(stime_t e, const adevs_bag<PortValue>& input)
{
	for (int i = 0; i < input.getSize(); i++)
	{
		// For each neighbor that is burning (or cooling), alter the heat level
		heat = heat + input.get(i).value;
	}
	// If the cell is on fire
	if (heat >= 2.0 && phase == UNBURNT && fuel > 0.0)
	{
		// Set the phase to burning and tell the neighbors that it is lit up
		phase = BURNING;
		hold(1.0);
	}
}

void fireCell::delta_conf(const adevs_bag<PortValue>& x) 
{
	delta_int();
	delta_ext(0,x);
}

void fireCell::output_func(adevs_bag<PortValue>& y)
{
	// Place the cell phase on the output port out.
	// The phase will be delivered to all neighboring cells.
	if (phase == BURNING)
	{
		output(out,1.0,y);
	}
	else if (phase == BURNT)
	{
		output(out,-1.0,y);
	}
}

void fireCell::gc_output(adevs_bag<PortValue>& g)
{
}

void fireCell::report(ostream& stream) const
{
	if (phase == BURNING || (phase == BURNT && fuel > 0.0))
		stream << "2 ";
	else if (phase == BURNT)
		stream << "1 ";
	else if (phase == UNBURNT)
		stream << "0 ";
}

fireCell::~fireCell()
{
}
