#ifndef __fireCell_h_
#define __fireCell_h_
#include <adevs.h>
#include <iostream>

/**
The phase of the cell.
*/
typedef enum { BURNING, BURNT, UNBURNT } CellPhase;

/**
A cell in the simple forest fire model.  The cell catches fire
when at least two of its neighbors are burning.  The cell continues
to burn until it is out of fuel.
*/
class fireCell: public atomic, public cell_interface
{
	public:
		// Constructor
		fireCell();
		// State initialization function
		void init();
		// Internal transition function
		void delta_int();
		// External transition function
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		// Confluent transition function
		void delta_conf(const adevs_bag<PortValue>& x);
		// Output function
		void output_func(adevs_bag<PortValue>& y);
		// Garbage collection
		void gc_output(adevs_bag<PortValue>& g);
		// Reports the state of the cell to the provided output stream
		void report(std::ostream& stream) const;
		// Destructor
		~fireCell();
		// Set the initial data for the cell
		void set_init_conditions(double fuel, bool burning);

	protected:
		// The phase of the cell
		CellPhase phase;
		// Amount of fuel available in the cell
		double fuel;
		// Number of burning neighbor cells
		double heat;
};

#endif
