#include "fireCell.h"
#include "fireCellWind.h"
#include "configuration.h"
#include <iostream>
#include <fstream>
#include <stdio.h>
using namespace std;

int main(int argc, char** argv)
{
	if (argc < 2)
	{
		std::cerr << argv[0] << " <configuration file>" << std::endl;
		std::cerr << "No input file" << std::endl;
		return 0;
	}
	// Load the initial fire model data
	configuration* config = new configuration(argv[1]);
	// Create a Moore-style cellspace.  
	// Each cell is influenced by the eight cells
	// touching it (in 3D this means 26 points, in 2D its only 8).
	cellSpace* space = new cellSpace(cellSpace::TWENTY_SIX_POINT,
		config->get_width(),config->get_height());
	// Set the wind direction
	fireCellWind::setWindDirection(config->get_wind());
	// Create an atomic cell to go into each point of the cellspace
	for (int x = 0; x < config->get_width(); x++)
	{
		for (int y = 0; y < config->get_height(); y++)
		{
			fireCellWind* fire_cell = new fireCellWind();
			fire_cell->set_init_conditions(config->get_fuel(x,y),
				config->get_fire(x,y));
			space->add(fire_cell,x,y);
		}
	}
	// Get simulation end time
	int tend = config->get_tend();
	// Done!
	delete config;
	// Run the simulation
	cout << "Running..." << endl;
	devssim sim(space);
	sim.run(tend);
	// Save the results to a file
	ofstream log_file("soln");
	for (int y = 0; y < space->getHeight(); y++)
	{
		for (int x = 0; x < space->getWidth(); x++)
		{
			const fireCell* fire_cell=dynamic_cast<const fireCell*>(space->getModel(x,y));
			fire_cell->report(log_file);
		}
		log_file << endl;
	}
	log_file.close();
	// Done!
	delete space;
	return 0;
}
