#ifndef __simple_atomic_h_
#define __simple_atomic_h_
#include "adevs.h"
#include <iostream>

/**
Simple atomic model that will be moved about the network.
*/
class simple_atomic: public atomic
{
	public:
		/**
		Create an atomic model that will undergo periodic internal events,
		up to the specified limit.
		*/
		simple_atomic(double event_interval, int event_limit = 1):
		atomic(),
		event_interval(event_interval),
		event_limit(event_limit)
		{
		}
		/// Atomic model initialization method
		void init()
		{
			hold(event_interval);
		}
		/// Atomic internal transition function
		void delta_int()
		{
			event_limit--;
			std::cout << "Simple atomic event at " << timeCurrent() << std::endl;
			if (event_limit > 0)
			{
				hold(event_interval);
			}
			else
			{
				passivate();
			}
		}
		/// Atomic external transition function
		void delta_ext(ADEVS_TIME_TYPE e, const adevs_bag<PortValue>& x)
		{
			// continue with next event time unchanged
		}
		/// Atomic confluent transition function
		void delta_conf(const adevs_bag<PortValue>& x)
		{
			delta_int();
		}
		/// Atomic output function
		void output_func(adevs_bag<PortValue>& y)
		{
			output(0,new object(),y);
		}
		/// Output garbage collection function
		void gc_output(adevs_bag<PortValue>& g)
		{
			for (int i = 0; i < g.getSize(); i++)
			{
				delete g[i].value;
			}
		}

	private:
		double event_interval;
		int event_limit;
};

#endif	

			
