#include "phold_network.h"
#include "perf_timer.h"
#include <ctime>
#include <iostream>
using namespace std;

#define INITIAL_NODES 100
#define INITIAL_TOKENS 10
#define STRUCTURE_CHANGE_INTERVAL ADEVS_INFINITY
#define RUNNING_TIME 1000.0

int main()
{
	try
	{
		// Seed the random number generator (everything in this model is based on rand())
		unsigned long seed = time(NULL);
		cout << "srand(" << seed << ")" << endl;
		srand(seed);
		// Create the top level network 
		phold_network network(INITIAL_NODES,INITIAL_TOKENS,STRUCTURE_CHANGE_INTERVAL);
		// Run the model
		devssim sim(&network);
		tic();
		sim.run(RUNNING_TIME);
		double t_run = toc();
		// Dump the performance stats
		cout << "INITIAL_NODES = " << INITIAL_NODES << endl;
		cout << "INITIAL_TOKENS = " << INITIAL_TOKENS << endl;
		cout << "STRUCTURE_CHANGE_INTERVAL = " << STRUCTURE_CHANGE_INTERVAL << endl;
		cout << "RUNNING_TIME = " << RUNNING_TIME << endl;
		cout << "Running time was " << t_run << " msec." << endl;
		cout << "Phold queue event count was " << phold_node::event_count << endl;
		cout << "Number of models created " << phold_node::models_created << endl;
		cout << "Number of models destroyed " << phold_node::models_destroyed << endl;
		cout << "Avg. event rate was  " << 
			(double)phold_node::event_count/(double)(t_run/1000.0) << " events/sec." << endl;
	}
	catch(adevs_exception err)
	{
		cerr << "Simulation abort: " << err << endl;
	}
	return 0;
}
