#include "rumorMonger.h"
#include <algorithm>
#include <ctime>
#include <cassert>
using namespace std;

const double rumorMonger::MAX_RUMOR_STR = 100.0;
rv rumorMonger::amp(time(NULL));
const port_t rumorMonger::gossip_in(0);
const port_t rumorMonger::gossip_out(1);

rumorMonger::rumorMonger(double rumor, double* output_mirror):
atomic(),
rumor(rumor),
threshold(0.0),
output_mirror(output_mirror)
{
}

void rumorMonger::init()
{
	/// Start the rumor or wait to hear something
	if (rumor > 0.0)
	{
		hold(1.0);
	}
	else
	{
		passivate();
	}
	set_mirror(rumor);
}

void rumorMonger::delta_int()
{
	// If we spread the rumor, then stop gabbing and only pass on
	// a rumor that is as good as the last one.
	threshold = rumor;
	rumor = 0.0;
	passivate();
}

void rumorMonger::delta_ext(stime_t e, const adevs_bag<PortValue>& x)
{
	// Take the best rumor and pass it on if it is sufficiently good.  
	// The individual will randomly help to "build the hype"
	// or "play it down", resulting in either an increase or decrease
	// in the strength of the rumor.
	for (int i = 0; i < x.getSize(); i++)
	{
		dbl gossip = *(dynamic_cast<const dbl*>(x.get(i).value));
		rumor = max(rumor,gossip.value*amp.uniform(0.0,1.4));
	}
	// The appetite for gossip increases linearly with time, until
	// the threshold reaches zero and the individual will gossip
	// about anything.
	threshold = max(0.0,threshold-e);
	// Pass on the rumor if it exceeds the individual's gossip threshold.
	if (rumor > threshold)
	{
		hold(1.0);
	}
	else
	{
		passivate();
	}
}

void rumorMonger::delta_conf(const adevs_bag<PortValue>& x)
{
	delta_int();
	delta_ext(0.0,x);
}

void rumorMonger::output_func(adevs_bag<PortValue>& y)
{
	double rumor_out = min(rumor,MAX_RUMOR_STR);
	output(gossip_out,new dbl(rumor_out),y);
	set_mirror(rumor_out);
}

void rumorMonger::gc_output(adevs_bag<PortValue>& g)
{
	for (int i = 0; i < g.getSize(); i++)
	{
		delete g[i].value;
	}
}

void rumorMonger::set_mirror(double value)
{
	if (output_mirror != NULL)
	{
		*output_mirror = value;
	}
}

rumorMonger::~rumorMonger()
{
}
