#ifndef __rumorMonger_h_
#define __rumorMonger_h_
#include "adevs.h"

class rumorMonger: public atomic 
{
	public:
		// Maximum strength of a rumor.
		static const double MAX_RUMOR_STR;

		/**
		Constructor.  The initial rumor value should be > 0 if
		this individual will spread the rumor.
		*/
		rumorMonger(double rumor = 0.0, double* output_mirror = NULL);
		/// State initialization function.
		void init();
		/// Internal transition function.
		void delta_int();
		/// External transition function.
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		/// Confluent transition function.
		void delta_conf(const adevs_bag<PortValue>& x);
		/// Output function.  
		void output_func(adevs_bag<PortValue>& y);
		/// Output value garbage collection.
		void gc_output(adevs_bag<PortValue>& g);
		/// Destructor.
		~rumorMonger();
		/// Model input port.
		static const port_t gossip_in;
		/// Model output port.
		static const port_t gossip_out;

	private:
		/**
		The rumor to spread.  The "strength" or "juiciness" of the
		rumor is indicated by the value.  Any rumor exceeding the 
		threshold will be passed on, otherwise it is discarded.
		*/
		double rumor;
		/// Rumor juiceness needed to cause the rumor to be passed on.
		double threshold;
		/// Random variable for rumor applification
		static rv amp;
		/// Mirror for output variable.  Used for visualization.
		double* output_mirror;
		void set_mirror(double value);
};

#endif
