#include "CDF.h"
#include "params.h"

CDF::CDF(bool left_edge):
atomic(),
cell_interface(),
left_boundary(false),
right_boundary(false),
left_edge(left_edge)
{
}

CDF::CDF(adevs_vec<3> ul, adevs_vec<3> ur, 
bool left_boundary, bool right_boundary, bool left_edge):
atomic(),
cell_interface(),
left_boundary(left_boundary),
right_boundary(right_boundary),
left_edge(left_edge)
{
	s.ul = ul;
	s.ur = ur;
}

CDF::CDF(const CDF& src):
atomic(src),
cell_interface(src),
s(src.s),
left_boundary(src.left_boundary),
right_boundary(src.right_boundary),
left_edge(src.left_edge)
{
}

const CDF& CDF::operator=(const CDF& src)
{
	atomic::operator=(src);
	cell_interface::operator=(src);
	s = src.s;
	left_boundary = src.left_boundary;
	right_boundary = src.right_boundary;
	left_edge = src.left_edge;
	return *this;
}

void CDF::init()
{
	compute_flux();
	hold(0.0);
}

void CDF::delta_int()
{
	passivate();
}

void CDF::delta_ext(stime_t e, const adevs_bag<PortValue>& x)
{
	delta_conf(x);
}

void CDF::delta_conf(const adevs_bag<PortValue>& x)
{
	for (int i = 0; i < x.getSize(); i++)
	{
		// If input came from the left
		if (x.get(i).port == cell_interface::in[0][1][1])
			s.ul = x.get(i).value;
		// If input came from the right
		else if (x.get(i).port == cell_interface::in[2][1][1])
			s.ur = x.get(i).value;
		// If the input is external to the cell block
		else if (left_edge)
			s.ul = x.get(i).value;
		else
			s.ur = x.get(i).value;
	}
	boundary_conditions();
	compute_flux();
	hold(0.0);
}

void CDF::output_func(adevs_bag<PortValue>& y)
{
	output(out,s.f,y);
}

void CDF::boundary_conditions()
{
	if (left_boundary)
	{
		s.ul[0] = s.ur[0];
		s.ul[1] = -s.ur[1];
		s.ul[2] = s.ur[2];
	}
	else if (right_boundary)
	{
		s.ur[0] = s.ul[0];
		s.ur[1] = -s.ul[1];
		s.ur[2] = s.ul[2];
	}
}

void CDF::compute_flux() 
{ 
	s.f = fluxes(s.ul,s.ur); 
}
