#ifndef __CDF_h_
#define __CDF_h_
#include <adevs.h>

/**
Computes the flux through a cell boundary.
*/
class CDF: public atomic, public cell_interface
{
	public:

		/**
		Output value is flux through the cell.
		Input values are left and right solutions.
		As a cell model, it assumes that grid is layed
		out along the x-axis.
		*/

		/// Create a CDF cell with initial value equal to 0
		CDF(bool left_edge = false);
		/// Create a CDF cell with initial soln values ul and ur
		CDF(adevs_vec<3> ul, adevs_vec<3> ur,
		bool left_boundary = false, bool right_boundary = false, bool left_edge = false);
		/// Copy constructor
		CDF(const CDF& src);
		/// Create a copy of this cell
		object* clone() const { return new CDF(*this); }
		/// Assignment operator
		const CDF& operator=(const CDF& src);
		void init();
		void delta_int();
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		void delta_conf(const adevs_bag<PortValue>& x);
		void output_func(adevs_bag<PortValue>& y);
		void gc_output(adevs_bag<PortValue>& y){}
		~CDF(){}

	private:
		struct state_t
		{
			adevs_vec<3> ul, ur, f;
			double tL, tN;
		};
		state_t s;
		bool left_boundary, right_boundary, left_edge;
		void compute_flux();
		void boundary_conditions();
};

#endif
