#include "cell.h"
#include <algorithm>
using namespace std;

double cell::ta_min = ADEVS_INFINITY;
double cell::dx = 1.0;
double cell::D[3] = { 0.0,0.0,0.0 };

cell::cell(bool left_edge):
atomic(),
cell_interface(),
left_edge(left_edge)
{
}

cell::cell(adevs_vec<3> u0, bool left_edge):
atomic(),
cell_interface(),
left_edge(left_edge)
{
	for (unsigned i = 0; i < 3; i++)
	{
		s.q.setParams(i,D[i]);
		s.q.init(i,u0[i]);
	}
}

cell::cell(const cell& src):
atomic(src),
cell_interface(src),
s(src.s),
left_edge(src.left_edge)
{
}

const cell& cell::operator=(const cell& src)
{
	atomic::operator=(src);
	cell_interface::operator=(src);
	s = src.s;
	left_edge = src.left_edge;
	return *this;
}

void cell::init()
{
	dddu = du_sum = 0.0;
	delta_count = 0;
	hold(s.q.ta());
	ta_min = min(s.q.ta(),ta_min);
}

void cell::delta_int()
{
	du_sum += dddu*ta();
	delta_count++;
	s.q.integ(s.q.ta());
	hold(s.q.ta());
	ta_min = min(s.q.ta(),ta_min);
}

void cell::delta_ext(stime_t e, const adevs_bag<PortValue>& x)
{
	du_sum += dddu*ta();
	delta_count++;
	for (int i = 0; i < x.getSize(); i++)
	{
		/// Left boundary flux
		if (x.get(i).port == cell_interface::in[0][1][1])
			s.fl = x.get(i).value;
		/// Right boundary flux
		else if (x.get(i).port == cell_interface::in[2][1][1])
			s.fr = x.get(i).value;
		/// External input
		else if (left_edge)
			s.fl = x.get(i).value;
		else
			s.fr = x.get(i).value;
	}
	adevs_vec<3> du((s.fl-s.fr)/dx);
	dddu = fabs(du.max());
	s.q.integ(e,*du);
	hold(s.q.ta());
}

void cell::delta_conf(const adevs_bag<PortValue>& x)
{
	delta_ext(s.q.ta(),x);
	ta_min = min(s.q.ta(),ta_min);
}

void cell::output_func(adevs_bag<PortValue>& y)
{
	adevs_vec<3> yy;
	for (int i = 0; i < 3; i++)
	{
		yy[i] = s.q.out(i);
	}
	output(cell_interface::out,yy,y);
}

adevs_vec<3> cell::getState() const
{
	adevs_vec<3> ss;
	for (int i = 0; i < 3; i++)
		ss[i] = s.q.state(i);
	return ss;
}
