#ifndef __cell_h_
#define __cell_h_
#include <adevs.h>

/**
A solution point.
*/
class cell: public atomic, public cell_interface
{
	public:
	
		/**
		Cell input is a left and right flux.  Cell output
		is the solution value.  Assumes that the solution
		grid is on the x-axis.
		*/
		/// Weighted sum of derivatives used in calculation [sum(ei*dui)]
		double du_sum, dddu;
		/// State change count
		unsigned long delta_count;
		/// Smallest observed time advance
		static double ta_min;
		/// Grid resolution
		static double dx;
		/// Quantum sizes
		static double D[3];

		/// Create a cell with a undefined initial conditions
		cell(bool left_edge = false);
		/// Create a cell with initial value u0
		cell(adevs_vec<3> u0, bool left_edge = false);
		/// Copy constructor
		cell(const cell& src);
		/// Create a copy of this cell
		object* clone() const { return new cell(*this); }
		/// Assignment operator
		const cell& operator=(const cell& src);
		void init();
		void delta_int();
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		void delta_conf(const adevs_bag<PortValue>& x);
		void output_func(adevs_bag<PortValue>& y);
		void gc_output(adevs_bag<PortValue>& y){}
		~cell(){}
		adevs_vec<3> getState() const;
	private:
		struct state_t
		{
			qoden<3> q;
			adevs_vec<3> fl, fr;
			double tL, tN;
		};
		state_t s;
		// Is this cell on the left edge of a cellblock boundary?
		bool left_edge;
};

#endif
