/* test.c ****************************************************\
*                                                             *
*  This program performs test of the                          *
*  exact Newton-type iterative Riemann solver of Godunov      *
*                                                             *
*  written by  Andrei A. Chernousov                           *
*  E-mail: <andrei99@iname.com>                               *
*  http://www.geocities.com/andrei_chernousov                 *
*  November 23, 2000                                          *
**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*--- exact Newton-type iterative Riemann solver of Godunov ---*/
#include "solver.c"

/* testing of Godunov() procedure */
int main( )
{
float
   /* relative velocity */
   V, Vmax, Vstep,
   /* primitive variables */
   R, U, P,
   R_1, U_1, P_1,
   R_2, U_2, P_2;

unsigned LEN;

FILE *pF;

   /* specific Riemann problem */
   R_1 = 2.0, U_1 = 0., P_1 = 200000.;
   R_2 = 1.0, U_2 = 0., P_2 = 100000.;

   /* */
   Vmax = 500.;
   LEN =  800.;
   Vstep = 2 * Vmax / LEN;

   /* output to "input_t.dat" file */
   if ( ( pF = fopen( "input_t.dat", "w" ) ) == NULL ) {
	  puts( "Cannot open \"input_t.dat\"" );
	  exit( -1 );
   }
   fprintf( pF, "%d\n%d\n", LEN+2, 3 );
   fprintf( pF, "%f 90000. 0.9 0. \n", -Vmax-0.00001 );
   for ( V = Vmax; V > -Vmax; V -= Vstep ) {
	  /* exact Riemann solver of Godunov */
	  Godunov( R_1, U_1+V, P_1,  R_2, U_2+V, P_2,  &R,  &U,  &P );
	  fprintf( pF,"%3.1f %6.0f %1.4f %3.1f \n", -V, P, R, U-V );
   }
   fprintf(pF,"%f 210000. 2.1 100. \n", Vmax+0.00001 );
   fclose( pF );

   return 0;

} /* end main() */

/*--- end test.c ---*/