#include "configuration.h"
#include "params.h"
#include <vector>
#include <iostream>
#include <fstream>
using namespace std;

configuration::configuration(string filename)
{
	// loading data from file configuration file
	ifstream pF (filename.c_str());
	if(pF == 0) 
	{
		throw string("Cannot open configuration file");
   }
	while (!pF.eof())
	{
		string key;
		pF >> key;
		if (key == "length") pF >> length;
		else if (key == "dx") pF >> dx;
		else if (key == "dt") pF >> dt;
		else if (key == "tend") pF >> tend;
		else if (key == "D0") pF >> D[0];
		else if (key == "D1") pF >> D[1];
		else if (key == "D2") pF >> D[2];
	} 
	pF.close();
	// generate initial conditions
	int i;
	adevs_vec<3> u;
	double U, R, P;
	for( i = 1, U = 0., R = 2., P = 200000.; i <= length/2; i++ ) 
	{
		u[0] = R;
		u[1] = R * U;
		u[2] = P / K_1 + 0.5 * R * U * U;
		data.push_back(u);
	}
   for( U = 0., R = 1., P = 100000.; i < length+1; i++ ) 
	{
		u[0] = R;
		u[1] = R * U;
		u[2] = P / K_1 + 0.5 * R * U * U;
		data.push_back(u);
	}
	// left boundary conditions 
	u[0] = data[0].get(0);
	u[1] = -1.0 * data[0].get(1);
	u[2] = data[0].get(2);
	data.insert(data.begin(),u);
	// right boundary conditions 
	u[0] = data[data.size()-1].get(0);
	u[1] = -1.0 * data[data.size()-1].get(1);
	u[2] = data[data.size()-1].get(2);
	data.push_back(u);
	length = data.size();
}
