#ifndef _configuration_h_
#define _configuration_h_
#include <vector>
#include <string>
#include "adevs_vec.h"

class configuration
{
	public:
		configuration(std::string filename);
		// Get the number of cells, including boundary cells
		int get_length() const
		{
			return length;
		}
		// Get the spatial resolution
		double get_dx() const
		{
			return dx;
		}
		// Get the integration time step
		double get_dt() const
		{
			return dt;
		}
		// Get the computation end time
		double get_tend() const
		{
			return tend;
		}
		// Get the integration quantum for the ith state variable
		double get_quantum(int i) const
		{
			return D[i];
		}
		// Get the ith data point (0 and get_length()-1 are the boundaries)
		adevs_vec<3> get_data(int i) const
		{
			return data.at(i);
		}
		// Get a reference to the data set
		std::vector<adevs_vec<3> >& get_data() 
		{
			return data;
		}
		~configuration()
		{
		}
	private:
		int length;
		double dx, dt, tend;
		double D[3];
		std::vector<adevs_vec<3> > data;
};

#endif
