#include "params.h"
#include "cell.h"
#include "CDF.h"
#include "configuration.h"
#include <vector>
#include <iostream>
#include <fstream>
#include <string>
#include "perf_timer.h"
using namespace std;

static double tend;

static void report(cellSpace* space)
{
	static unsigned long freq = 0;
	static ofstream stream("soln");
	freq = (freq+1)%1000;
	if (freq != 1)
	{
		return;
	}
	float index = 1.0;
	for (int x = 0; x < space->getWidth(); x++)
	{
		const cell* c = dynamic_cast<const cell*>(space->getModel(x));
		if (c != NULL)
		{
			adevs_vec<3> s(c->getState());
			double R = s[0];
			double U = s[1] / R;
			double P = (s[2] - 0.5 * R * U * U ) * K_1;
//			stream << P << " " << R << " " << U << endl;
			stream << s[0] << " " << s[1] << " " << s[2] << " " << c->delta_count << endl;
//			stream << 1.0/c->ta() << " ";
			index += 1.0;
		}
	}
	stream << endl;
}

static cellSpace* make_model(string filename)
{
	configuration config(filename);
	vector<devs*> models;
	cell::dx = config.get_dx();
	for (int i = 0; i < 3; i++)
	{
		cell::D[i] = config.get_quantum(i);
	}
	tend = config.get_tend();
	unsigned index = 0;
	vector<adevs_vec<3> >::const_iterator i = config.get_data().begin();
	vector<adevs_vec<3> >::const_iterator k = config.get_data().begin(); k++;
	// left boundary and interior and right boundary
	for (; index < config.get_data().size()-1; index++,i++,k++)
	{
		models.push_back(new CDF(*i,*k,index==0,index==(config.get_data().size()-2)));
		if (index < config.get_data().size()-2)
		{
			models.push_back(new cell(*k));
		}
	}
	cellSpace* cells = new cellSpace(cellSpace::SIX_POINT,models.size());
	for (index = 0; index < models.size(); index++)
	{
		cells->add(models[index],index);
	}
	return cells;
}

int main(int argc, char** argv) 
{
	try 
	{
		cellSpace* space = make_model(argv[1]);
		xdevssim sim(space);
		sim.initialize();
		tic();
		while (sim.nextTN() < tend)
		{
			sim.deltFunc(sim.nextTN());
		}
		double e = toc();
		cout << e << " ms.\nmin_ta = " << cell::ta_min << endl;
		report(space);
		sim.finalize();
		delete space;
	}
	catch(adevs_exception& err) 
	{
		cerr << err.getErrorMsg() << endl;
	}
	catch(string& err)
	{
		cerr << err << endl;
	}
	return 0;
}


