/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __ball_h_
#define __ball_h_
#include <adevs.h>
#include "circle.h"
#include "rectangle.h"
#include "events.h"
/**
Model of the ball.
*/
class ball: public atomic 
{
	public:
		ball();
		// construct ball with initial velocity
		ball(const vec& v_init);
		// construct ball with initial shape
		ball(const circle& geom_init);
		// construct ball with shape and velocity
		ball(const circle& geom_init, const vec& v_init);

		// set the bounding box for the ball's motion
		void setWorld(const rectangle& world_size);
		void setWorld(double x, double y, double width, double height);

		void init();
		void delta_int();
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		void delta_conf(const adevs_bag<PortValue>& x);
		void output_func(adevs_bag<PortValue>& y);
		void gc_output(adevs_bag<PortValue>& g);

		~ball(){}

		// input ports
		static const port_t collision;
		static const port_t stop;

		// output ports
		static const port_t motion; 
		static const port_t destroyed;

	private:
		// constants
		static const double MOTION_RES;
		// initial state
		eStatus status;
		vec v_init;
		circle geom_init;
		circle geom;
		circle geom_op;
		// state
		vec v;
		vec dp;
		rectangle world_size;

		void checkStatus();
};

#endif
