/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __block_h_
#define __block_h_
#include <adevs.h>
#include <list>
#include "rectangle.h"
#include "events.h"
/**
Block's are used to represent walls and, um, blocks.
*/
class block: public atomic 
{
	public:
		block();
		// create a block with a given shape.
		block(const rectangle& geom_init);
		// create a block with a shape and fixed status (normally INVULNERABLE).
		block(const rectangle& geom_init, eStatus stat);

		void init();
		void delta_int();
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		void delta_conf(const adevs_bag<PortValue>& x);
		void output_func(adevs_bag<PortValue>& y);
		void gc_output(adevs_bag<PortValue>& g);

		~block(){}

		// input ports
		static const port_t ball_motion;
		static const port_t paddle_motion;

		// output ports
		static const port_t status; 
		static const port_t ball_collision;
		static const port_t paddle_collision;

	private:
		bool fixed;
		rectangle geom_init;
		rectangle geom;
		eStatus stat_init;
		eStatus stat;
		bool report_status;
		std::list<object*> ball_collisions;
		std::list<object*> paddle_collisions;
};

#endif
