/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <stdlib.h>
#include <math.h>
#include "shape.h"
#include "circle.h"
#include "rectangle.h"

bool circle::contains(double x, double y) const 
{
	vec v(getPosition().x - x, getPosition().y - y);
	return (v.length() <= radius);
}

bool circle::contains(const vec& pos) const 
{
	vec v(getPosition() - pos);
	return (v.length() <= radius);
}

double circle::surfaceAngle(double x, double y) const 
{
	vec p(x-getPosition().x,y-getPosition().y);
	return asin (p.y / p.length ());
}

double circle::surfaceAngle(const vec& pos) const 
{
	vec p(pos - getPosition ());
	return asin (p.y / p.length ());
}

bool circle::overlaps(const circle& c) const 
{
	vec l(c.getPosition() - getPosition ());
	return (l.length () < (c.getRadius()+radius));
}

bool circle::overlaps(const rectangle& r) const 
{
	return r.overlaps(*this);
}
