/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __circle_h_
#define __circle_h_
#include "shape.h"
/**
A 2D circle.
*/
class circle: public shape 
{
	public:
		circle(double radius = 1.0):shape(),radius(radius){}
		circle(double x, double y, double radius = 1.0):shape(x,y),radius(radius){}
		circle(const vec& pos, double radius = 1.0):shape(pos),radius(radius){}

		double getRadius() const { return radius; }
		void setRadius(double radius) { this->radius = radius; }

		// returns true if the points is in the circle, false otherwise.
		virtual bool contains(double x, double y) const;
		virtual bool contains(const vec& pos) const;

		// returns the angle of the tangent line relative to the x axis
		virtual double surfaceAngle(double x, double y) const;
		virtual double surfaceAngle(const vec& pos) const;

		// overlap is determined by rectangular approximation.
		virtual bool overlaps(const circle& c) const;
		virtual bool overlaps(const rectangle& r) const;

		virtual ~circle(){}

	private:
		double radius;
};

#endif
