/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __events_h_
#define __events_h_
#include <adevs.h>
#include "shape.h"
/**
Events that are generated/received by the DEVS models.
*/

// Indicates a collision between the target object and the indicated shape.
class collisionEvent: public object 
{
	public:
		collisionEvent(const object* target, const shape* geom):
		object(),target(target),geom(geom){} 
		const object* getTarget() const { return target; }
		const shape* getGeometry() const { return geom; }
	
	private:
		const object* target;
		const shape* geom;
};

// Indicates significant motion by the source object.
class motionEvent: public object 
{
	public:
		motionEvent(const object* source, const vec& velocity,
		const shape* geom):
		object(),source(source),velocity(velocity),geom(geom){}
		motionEvent(const object* source, const shape* geom):
		object(),source(source),geom(geom){}
		const object* getSource() const { return source; }
		const shape* getPosition() const { return geom; }
		const vec& getVelocity() const { return velocity; }

	private:
		const object* source;
		vec velocity;
		const shape* geom;
};

// Used to report the status of the source object.
enum eStatus { OK, DESTROYED, INVULNERABLE };
class statusEvent: public object 
{
	public:
		statusEvent (const object* source, eStatus status, 
		const shape* geom):
		object(),source(source),status(status),geom(geom){}
		eStatus getStatus() const { return status; }
		const object* getSource() const { return source; }
		const shape* getGeometry() const { return geom; }

	private:
		const object* source;
		eStatus status;
		const shape* geom;
};

// A paddle control event (generated by the player).
enum ePaddleControl { LEFT, RIGHT, STOP };
class paddleControlEvent: public object 
{
	public:
		paddleControlEvent(ePaddleControl sig):
		object(),sig(sig){}
		ePaddleControl getControl() const { return sig; }

	private:
		ePaddleControl sig;
};

#endif
