/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __paddle_h_
#define __paddle_h_
#include <adevs.h>
#include <list>
#include "rectangle.h"
/**
Model of the paddle.
*/
class paddle: public atomic 
{
	public:
		// constructors
		paddle();
		// create a paddle with initial shape
		paddle(const rectangle& geom_init);
		// create a paddle with the indicated velocity (assume v.y = 0 for now).
		paddle(const vec& v_paddle);
		// create a paddle with the indicated shape and velocity.
		paddle(const rectangle& geom_init, const vec& v_paddle);

		void init();
		void delta_int();
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		void delta_conf(const adevs_bag<PortValue>& x);
		void output_func(adevs_bag<PortValue>& y);
		void gc_output(adevs_bag<PortValue>& g);

		~paddle(){}

		// input ports
		static const port_t collision;
		static const port_t vcontrol;
		static const port_t ball_motion;
		static const port_t stop;

		// output ports
		static const port_t motion; 
		static const port_t ball_collision;

	private:
		// constants
		static const double MOTION_RES;
		static const vec V_ZERO;

		vec v_paddle;
		rectangle geom_init;
		rectangle geom;
		rectangle geom_op;
		std::list<object*> collisions;
		vec v;
		vec dp;
};

#endif
