/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <math.h>
#include <stdlib.h>
#include "shape.h"
#include "rectangle.h"
#include "circle.h"
#include "physics.h"

bool rectangle::contains(double x, double y) const 
{
	return (x >= getPosition().x && x <= getPosition().x+width && 
	y >= getPosition().y && y <= getPosition().y+height);
}

bool rectangle::contains(const vec& pos) const 
{
	return (pos.x >= getPosition().x && pos.x <= getPosition().x+width && 
	pos.y >= getPosition().y && pos.y <= getPosition().y+height);

}

double rectangle::surfaceAngle(double x, double y) const 
{ 
	double dist_left_side = fabs (x - getPosition().x);
	double dist_right_side = fabs (x - (getPosition().x + width));
	double dist_top_side = fabs (y - getPosition().y);
	double dist_bot_side = fabs (y - (getPosition().y+height));
	double dist_h, dist_v;
	if (dist_left_side < dist_right_side) dist_v = dist_left_side;
	else dist_v = dist_right_side;
	if (dist_top_side < dist_bot_side) dist_h = dist_top_side;
	else dist_h = dist_bot_side;

	if (dist_h == dist_v) return (PI / 4.0);
	else if (dist_h < dist_v) return 0;
	else return (PI / 2.0);
}

double rectangle::surfaceAngle(const vec& pos) const 
{
	return surfaceAngle(pos.x, pos.y);
}

/**
Draws a rectangle around the circle, then checks for overlap
with the resulting rectangel.  Good approximation for a small
circle.
*/
bool rectangle::overlaps (const circle& c) const 
{
	rectangle r(c.getPosition().x - c.getRadius(), c.getPosition().y - c.getRadius(),
	c.getRadius() * 2.0, c.getRadius () * 2.0);
	return overlaps(r);
}

/**
Looks for overlapping sides of the two rectangels.
*/
bool rectangle::overlaps(const rectangle& r) const 
{
	vec p1(r.getPosition());
	vec p2(r.getPosition().x, r.getPosition().y+r.getHeight());
	vec l1(getPosition());
	vec l2(getPosition().x+getWidth(),getPosition().y);

	// r left vs. top
	if (p1.x >= l1.x && p1.x <= l2.x && p1.y <= l1.y && p2.y >= l1.y)
		return true;

	// r right vs. top
	p1.x += r.getWidth (); 
	p2.x += r.getWidth ();
	if (p1.x >= l1.x && p1.x <= l2.x && p1.y <= l1.y && p2.y >= l1.y)
		return true;

	p1.x -= r.getWidth ();
	p2.x -= r.getWidth ();
	l1.y += getHeight ();
	l2.y += getHeight ();

	// r left vs. bottom
	if (p1.x >= l1.x && p1.x <= l2.x && p1.y <= l1.y && p2.y >= l1.y)
		return true;

	// r right vs. bottom 
	p1.x += r.getWidth (); 
	p2.x += r.getWidth ();
	if (p1.x >= l1.x && p1.x <= l2.x && p1.y <= l1.y && p2.y >= l1.y)
		return true;

	// r top vs. left
	p1 = r.getPosition ();
	p2.x = r.getPosition().x + r.getWidth ();
	p2.y = r.getPosition().y;
	l1 = getPosition ();
	l2.x = getPosition().x;
	l2.y = getPosition().y + getHeight ();
	if (p1.x <= l1.x && p2.x >= l2.x && p1.y >= l1.y && p2.y <= l2.y)
		return true;

	// r top vs. right
	l1.x += getWidth ();
	if (p1.x <= l1.x && p2.x >= l2.x && p1.y >= l1.y && p2.y <= l2.y)
		return true;

	return false;
}
