/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __rectangle_h_
#define __rectangle_h_
#include "shape.h"
/**
A 2D rectangle.
*/

class rectangle: public shape 
{
	public:
		rectangle():shape(),width(1),height(1){}
		rectangle(const vec& pos):shape(pos),width(1),height(1){} 
		rectangle(double width,double height):shape(),width(width),height(height){}
		rectangle(double x, double y, double width, double height):
		shape (x,y),width(width),height(height){}
		rectangle(const vec& pos, double width, double height):
		shape(pos),width(width),height(height){}

		double getWidth() const { return width; }
		double getHeight() const { return height; }
		void setSize(double width, double height) 
		{
			this->width = width; this->height = height; 
		}

		// return true of the rectangle contains the point, false otherwise
		virtual bool contains(double x, double y) const;
		virtual bool contains(const vec& pos) const;

		// returns the angle, relative to the x axis, that
		// is formed by the side closest to the point.
		virtual double surfaceAngle(double x, double y) const;
		virtual double surfaceAngle(const vec& pos) const;

		// approximates overlap by representing the circle as a rectangle
		virtual bool overlaps(const circle& c) const;
		virtual bool overlaps(const rectangle& r) const;

		virtual ~rectangle(){}

	private:
		double width, height;

};

#endif
