/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __shape_h_
#define __shape_h_
#include "vec.h"
/**
A 2D shape. 
*/

// predefine classes of interest for overlap testing.
class circle;
class rectangle;

class shape 
{
	public:
		// constructors
		shape(){}
		shape(double x, double y):anchor(x,y){}
		shape(const vec& position):anchor(position){}

		const vec& getPosition() const { return anchor; }

		// set the shape's anchor to (x, y)
		void setPosition (double x, double y) { anchor.x=x; anchor.y=y; }
		void setPosition (const vec& pos) { anchor=pos; }

		// adjust the shape's anchor position by (dx, dy)
		void deltaPos(double dx,double dy) { anchor.x+=dx; anchor.y+=dy; }
		void deltaPos(const vec& dp) { anchor += dp; }

		// return true if shape contains the point, false otherwise.
		virtual bool contains(double x, double y) const = 0;
		virtual bool contains(const vec& pos) const = 0;

		// return the angle of a line tangent to the point, relative to the
		// x axis.
		virtual double surfaceAngle(double x, double y) const = 0;
		virtual double surfaceAngle(const vec& pos) const = 0;

		// return true if the shapes overlap, false otherwise.
		virtual bool overlaps(const circle& c) const = 0;
		virtual bool overlaps(const rectangle& r) const = 0;

		virtual ~shape(){}

	private:
		vec anchor;
};

#endif
