/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __vec_h_
#define __vec_h_
/**
A vec in 2 dimensions.
*/
class vec 
{
	public:
		double x, y;

		// constructors
		vec():x(0.0),y(0.0){}
		vec(double x, double y):x(x),y(y){}
		vec(const vec& src):x(src.x),y(src.y){}
		// returns the length of the vec
		double length() const;
		// binary operators 	
		vec operator+(const vec& v) const 
		{
			return vec(v.x+x, v.y+y); 
		} 
		vec operator-(const vec& v) const 
		{
			return vec(x-v.x, y-v.y); 
		} 
		vec operator* (double x) const 
		{
			return vec(this->x*x, this->y*x); 
		} 
		vec operator/(double x) const 
		{
			return vec (this->x/x, this->y/x); 
		} 
		// rotation and reflection transformations
		vec& rotate(double radians);
		vec& reflect(double radians);
		// assignment operators
		vec& operator+=(const vec& v) 
		{
			x += v.x; y += v.y; return *this; 
		}
		vec& operator-=(const vec& v) 
		{
			x -= v.x; y -= v.y; return *this; 
		}
		vec& operator*=(double x) 
		{
			this->x *= x; this->y *= x; return *this; 
		}
		vec& operator/=(double x) 
		{
			this->x /= x; this->y /= x; return *this; 
		}
		vec& operator=(const vec& v) 
		{ 
			x = v.x; y = v.y; return *this; 
		}
		// equivalence testing
		bool operator==(const vec& v) const 
		{
			return (x == v.x && y == v.y); 
		} 
		bool operator!=(const vec& v) const 
		{
			return (x != v.x && y != v.y); 
		} 
		~vec(){}
};

vec operator*(double x, const vec& v);

#endif
