/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __viewer_h_
#define __viewer_h_
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <adevs.h>
#include <map>
#include "events.h"
#include "rectangle.h"
#include "circle.h"
/**
The game display and controller constructed as an atomic model.
*/
class viewer: public atomic 
{
	// utility class to hold position histories
	friend class __pos;
	public:
		// input ports
		static const port_t ball_motion;
		static const port_t block_position;
		static const port_t paddle_motion;
		static const port_t ball_status;

		// output ports
		static const port_t paddle_control;
		static const port_t stop; 

		// Creates a view window.  Calls exit() on failure.
		viewer(char** argv, int argc, int width, int height);

		void init();
		void delta_int();
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		void delta_conf(const adevs_bag<PortValue>& x);
		void output_func(adevs_bag<PortValue>& y);
		void gc_output(adevs_bag<PortValue>& g);

		bool hasQuit() const { return quit; }
		int getScore() const { return score; }

		// Closes the window.  Frees all display resources.
		~viewer();
	private:
		static const stime_t PERIOD;
		// Draws a circle centered on (x, y).
		void drawBall(const circle& c, __pos& old);
		// Draws a rectangle centered at (x, y) with appropriate dimensions.
		void drawBlock(const rectangle& r, eStatus status);
		// Draws a rectangle centered at (x, y) with appropriate dimensions.
		void drawPaddle(const rectangle& r, __pos& old);
		// Draws the score string in the upper left corner.
		void drawScore (int score, bool first = false);
		// Draws the balls remaining string  in the upper left corner.
		void drawBallCount (int balls, bool first = false);
		// Check the window event queue
		void checkEvents ();

		Display* display;
		int screen;
		Window win;
		GC grphc_cntxt;

		XSizeHints size_hints;
		XEvent event;
		long event_mask;

		std::map<const object*,__pos> pos_history;
		paddleControlEvent* control_event;
		int score;
		char last_score [20];
		char last_ball_count [20];
		int balls;
		bool quit;
		int block_count;
};

#endif
