/***********
Copyright (C) 1999  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include "viewer.h"
#include "paddle.h"
#include "block.h"
#include "ball.h"
#include <iostream>
#include <stdlib.h>
using namespace std;

/**
Top level model for the game.
*/

class xblock: public staticDigraph 
{
	public:
		xblock(int argc, char** argv);
		bool hasQuit() const { return display->hasQuit(); }
		int getScore() const { return display->getScore(); }
		~xblock();

	private:
		viewer* display;
};

xblock::xblock(int argc, char** argv):staticDigraph() 
{
	// number of blocks in game
	int block_count = 3 + 52;
	block* blocks [block_count];
	// block dimension (width and height)
	int block_dim = 30;
	// game board dimensions
	int width = 412;
	int height = 300;
	int border = 30;
	// set up walls
	rectangle north (border, border, width, 2);
	rectangle west (border, border, 2, height);
	rectangle east (width+border, border, 2, height);
	blocks[0] = new block (north, INVULNERABLE); 
	blocks[1] = new block (west, INVULNERABLE); 
	blocks[2] = new block (east, INVULNERABLE); 
	// place blocks
	rectangle line (border+5, border+5, block_dim, block_dim);
	for (int i = 3; i < block_count; i++) 
	{
		blocks[i] = new block (line);
		line.deltaPos (block_dim + 1, 0);
		if (line.getPosition().x+line.getWidth() >= border+width) 
			line.setPosition (border+5, line.getPosition().y + block_dim + 1);
	}
	// setup the paddle
	rectangle paddle_shape (width / 2, height, 18, 1.5);
	paddle* p = new paddle (paddle_shape);
	// setup the ball
	circle ball_shape (width / 2, height - 10, 1);	
	vec v ((double)random () / (double)RAND_MAX, -1);
	ball* b = new ball (ball_shape, v);
	b->setWorld (border, border, width+2*border+1, height+border+1);
	// create the viewer
	display = new viewer (argv, argc, width+2*border+1, 
				height+border+1);
	// add paddle, viewer, and ball
	add(p);
	add(display);
	add(b);
	// establish couplings
	couple (b, ball::motion, display, viewer::ball_motion);
	couple (b, ball::destroyed, display, viewer::ball_status);
	couple (display, viewer::stop, b, ball::stop);
	couple (p, paddle::motion, display, viewer::paddle_motion);
	couple (p, paddle::ball_collision, b, ball::collision);
	couple (b, ball::motion, p, paddle::ball_motion);
	couple (display, viewer::paddle_control, p, paddle::vcontrol);
	couple (display, viewer::stop, p, paddle::stop);
	for (int i = 0; i < block_count; i++) 
	{
		add (blocks[i]);
		couple (blocks[i], block::status, display, viewer::block_position);
		couple (blocks[i], block::ball_collision, b, ball::collision);
		couple (b, ball::motion, blocks[i], block::ball_motion);
	}
	couple (p, paddle::motion, blocks[1], block::paddle_motion);
	couple (p, paddle::motion, blocks[2], block::paddle_motion);
	couple (blocks[1], block::paddle_collision, p, paddle::collision);
	couple (blocks[2], block::paddle_collision, p, paddle::collision);
}

xblock::~xblock(){}

/**
Main function to start the simulator.
*/
int main(int argc, char** argv) 
{
	try 
	{
		xblock _xblock(argc, argv);
		devssim sim (&_xblock);
		while (!_xblock.hasQuit()) 
		{
			sim.reset ();
			sim.run ();
		}
		cout << "Game over! Final score: " << _xblock.getScore () << endl;
	}
	catch(adevs_exception& err) 
	{
		cout << err.getErrorMsg() << endl; 
	}
	return 0;
}
