/***********
Copyright (C) 1999, 2000, 2001, 2002  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __cell_h_
#define __cell_h_
#include <adevs.h>
#include "viewer.h"
/**
A cell in the Game of Life.  
*/
class cell: public atomic 
{
	public:		
		// Create a cell and set the initial state to dead or alive
		cell(const std::pair<int,int>& location, bool alive = false);
		// Provided for the benefit of the coupled model that represents the
		// cell space.  Returns a 2D point indicating the location of this model
		// in the world.
		const point& getLocation () { return loc; }
		// required methods.
		void delta_int();
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		void delta_conf (const adevs_bag<PortValue>& x);
		void init();
		void output_func(adevs_bag<PortValue>& y);
		void gc_output(adevs_bag<PortValue>& g){}

		// Value is +1 if cell is born, -1 if cell dies
		static const port_t u;
		// Increment the neighbor count by value received on port (+1 or -1)
		static const port_t n;
		// Value is location if neighbor count is 0
		static const port_t q;
		// Value is location of neighbor if cell is born
		static const port_t b;

		// viewer used to display the state of the cell.
		static viewer* view;

		~cell();

		// Increment the alive count, used when creating new cells
		void incrAlive () { nalive++; }

	private:	
		// location of the model in the 2D space.
		point loc;
		// phase can be alive or dead.
		bool alive;
		// number of living neighbors.
		int nalive; 

		bool canLive () const { return (nalive == 2 || nalive == 3); }
};

#endif
