/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <map>
/**
The cellspace is a network executive containing a grid of cells.  The cell space
wraps at the edges.
*/
class cellspace: public netExec 
{
	public:		
		// constructs a cellspace with the indicated width and height.
		cellspace(int width, int height);

		// The compute neighhbors method is called by the simulator when messages
		// appear on the output port of a cell.  Returns the cells that form the
		// event generating cell's Moore neighborhood (minus the cell itself).
		void route(const PortValue& pv, devs* src, adevs_bag<EventReceiver>& r);
		// Get all the cells in the space
		void getComponents(adevs_set<devs*>& c);

		void init();
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		void delta_int();
		void delta_conf(const adevs_bag<PortValue>& x);
		void output_func(adevs_bag<PortValue>& y);
		void gc_output(adevs_bag<PortValue>& g){}
		void gc_models(adevs_set<devs*>& models)
		{
			for (int i = 0; i < models.getSize(); i++)
				delete models[i];
		}
		~cellspace();

		static const port_t isolated;
		static const port_t birthed;

	private:	
		// a hashtable mapping points to cells.
		std::map<point,cell*> cells;
		// dimensions of the cellspace.
		int width, height;
		// method used to add a point to the cellspace.
		cell* add_cell(const point& loc, bool alive);
		void add_cell(cell* c);
		void remove_cell(cell* model);
		// maps x, y points into the torrodial cellspace.
		point map_point (int x, int y) const;
		void getAdj (const point& p, point* l);
};

class world: public devsn
{
	public:
		world(int width,int height):
		devsn(),nx(width,height){}
		netExec* getNetExec(){ return &nx; }
	private:
		cellspace nx;
};
