/***********
Copyright (C) 1999, 2000, 2002  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __viewer_h_
#define __viewer_h_
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/cursorfont.h>
#include <adevs.h>
#include "point.h"

class viewer
{
	public:		
		// Creates the a view window.  Throws adevs_exception on failure.
		viewer(char** argv, int argc);
		// Turns on/off the cell at point p.
		void setState(const point& p, bool alive);
		// Display parameters.
		int getWidth() const;
		int getHeight() const;
		// Blank the screen.
		void blank();
		// Clear the X event queue, look for activity.  Return true
		// if terminating activity on the display since last time, false
		// otherwise.  
		bool checkEvents();
		// Closes the window.  Frees all display resources.
		~viewer();

		static const int MINDIM;
		static const int DEF_DIM;

	private:	
		int cellh, cellw, dwidth, dheight, gwidth, gheight;
		double xscale, yscale;

		Display* display;
		int screen;
		Window win;
		GC deadgc;
		GC livegc;

		XSizeHints size_hints;
		XEvent event;
		long event_mask;

		bool screensaver;
		bool root;

		void parseCommand (const char* argv);
};

#endif
