/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2002 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __adevs_exceptions_h_
#define __adevs_exceptions_h_
#include "adevs_config.h"
#include <string>
#include <iostream>
#include <stdio.h>

/** Macro for throwing adevs_exceptions.  The err_msg should
be a string that will be the error message assigned to the 
exception which is thrown.
*/
#define THROW_BASE_EXCEPTION(err_msg) \
{ \
char __buffer[100]; \
sprintf(__buffer,"%s :: %s line %d",err_msg,__FILE__,__LINE__); \
ADEVS_NAMESPACE::adevs_exception __err(__buffer); \
throw __err; \
}

ADEVS_NAMESPACE_BEGIN

/**
The root exception class for all exceptions throw by the adevs library.
In general, an exception is thrown only when a fatal error is detected.
*/
class adevs_exception
{
	public:
		/// Constructor.  Creates an exception with an empty error string.
		adevs_exception()
		{
		}
		/// Copy constructor.
		adevs_exception(const adevs_exception& src):
		err_str(src.err_str)
		{
		}
		/// Assignment operator.
		const adevs_exception& operator=(const adevs_exception& src)
		{
			err_str = src.err_str;
			return *this;
		}
		/**
		Create an exception with the supplied error message.
		*/
		adevs_exception(std::string err_str):
		err_str(err_str)
		{
		}
		/// Destructor.
		virtual ~adevs_exception()
		{
		}
		/// Get the error message associated with this exception.
		std::string getErrorMsg() { return err_str; }
		/// Print the error message to an output stream.
		friend std::ostream& operator<<(std::ostream& strm, adevs_exception& err)
		{
			strm << err.err_str;
			return strm;
		}
	private:
		std::string err_str;
};

ADEVS_NAMESPACE_END

#endif
