/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2003 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __object_pool_h_
#include "adevs_config.h"
#include "adevs_bag.h"

ADEVS_NAMESPACE_BEGIN

/**
A utility class for managing pools of objects that are stored on the heap.
Uses the new and delete operators to create and destroy objects.
*/
template <class T> class object_pool
{
	public:
		// Construct an empty pool
		object_pool():
		pool()
		{
		}
		// Construct a pool with a specific capacity and initial population
		object_pool(int capacity, int pop = 0):
		pool(capacity)
		{
			for (int i = 0; i < pop; i++)
			{
				pool.add(new T());
			}
		}
		// Create an object 
		T* make_obj()
		{
			T* obj;
			if (pool.getSize() > 0)
			{
				obj = pool.get(0);
				pool.remove(0);
			}
			else
			{
				obj = new T;
			}
			return obj;
		}
		// Return an object to the pool
		void destroy_obj(T* obj)
		{
			pool.add(obj);
		}
		// Delete all objects in the pool
		~object_pool()
		{
			for (int i = 0; i < pool.getSize(); i++)
			{
				delete pool[i];
			}
		}
	private:
		adevs_bag<T*> pool;

		// No copy constructor
		object_pool(const object_pool<T>& src){}
		// No assignment operator
		void operator=(const object_pool<T>& src){}
};

ADEVS_NAMESPACE_END

#endif
