#ifndef __perf__timer_h
#define __perf__timer_h
#include "adevs_config.h"
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>

ADEVS_NAMESPACE_BEGIN

static struct timeval _t_start;
static struct timeval _t_end;

/**
Start the performance timer.  This only works on UN*X machines that support the
gettimeofday() function.
*/
static void tic()
{
	gettimeofday(&_t_start,NULL);
} 

/// Returns execution time in milli-seconds measured from the last call to tic().
static unsigned long toc()
{
	gettimeofday(&_t_end,NULL);
	unsigned long secs = _t_end.tv_sec - _t_start.tv_sec;
	unsigned long usecs;
	if (_t_end.tv_usec < _t_start.tv_usec)
	{
		secs--;
		usecs = 1000000L-(_t_start.tv_usec-_t_end.tv_usec);
	}
	else
	{
		usecs = _t_end.tv_usec-_t_start.tv_usec;
	}
	return secs*1000+ldiv(usecs,1000).quot;
}

/// Returns execution time in seconds from the last call to tic().
static unsigned long tocs()
{
	gettimeofday(&_t_end,NULL);
	unsigned long secs = _t_end.tv_sec - _t_start.tv_sec;
	return secs;
}

ADEVS_NAMESPACE_END

#endif
