/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2005 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutarojj@ornl.gov
***************/
#include "adevs_linux_numa_wrapper.h"
#include "adevs_config.h"
#ifdef ADEVS_USE_LINUX_NUMA
#include <cstdio>
#include <numa.h>
#endif

static bool check(int node)
{
	#ifdef ADEVS_USE_LINUX_NUMA
	if (numa_available() < 0)
	{
		fprintf(stderr,"numa not available\n",node);
		return false;
	}
	if (numa_max_node() < node && node != -1)
	{
		fprintf(stderr,"Bad node number %d, limit is %d!\n",node,numa_max_node());
		return false;
	}
	#endif
	return true;
}

void adevs_linux_numa_set_preferred(int node)
{
	#ifdef ADEVS_USE_LINUX_NUMA
	if (check(node))
	{
		numa_set_preferred(node);
	}
	#endif
}

void adevs_linux_numa_run_on_node(int node)
{
	#ifdef ADEVS_USE_LINUX_NUMA
	if (check(node))
	{
		numa_run_on_node(node);
	}
	#endif
}
