/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2003 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include "cellSpace_models.h"
using namespace std;
USING_NAMESPACE_ADEVS

#ifdef OK_TO_COMPILE_ADEVS_CELLSPACE

const int ADEVS_NAMESPACE::cellSpace::SIX_POINT = 0;
const int ADEVS_NAMESPACE::cellSpace::TWENTY_SIX_POINT = 1;

const ADEVS_PORT_TYPE ADEVS_NAMESPACE::cell_interface::in[3][3][3] = 
{
	{ { 0 , 1, 2 }, { 3, 4, 5 }, { 6, 7, 8 } },
	{ { 9, 10, 11 }, { 12, 13, 14 }, { 15, 16, 17 } },
	{ { 18, 19, 20 }, { 21, 22, 23 }, { 24, 25, 26 } }
};
const ADEVS_PORT_TYPE ADEVS_NAMESPACE::cell_interface::dout[3][3][3] = 
{
	{ { 0 , 1, 2 }, { 3, 4, 5 }, { 6, 7, 8 } },
	{ { 9, 10, 11 }, { 12, 13, 14 }, { 15, 16, 17 } },
	{ { 18, 19, 20 }, { 21, 22, 23 }, { 24, 25, 26 } }
};
const ADEVS_PORT_TYPE ADEVS_NAMESPACE::cell_interface::out = 27;

const int ADEVS_NAMESPACE::cell_interface::offset_table[28][3] =
{
	{-1,-1,-1},
	{-1,-1,0},
	{-1,-1,1},
	{-1,0,-1},
	{-1,0,0},
	{-1,0,1},
	{-1,1,-1},
	{-1,1,0},
	{-1,1,1},
	{0,-1,-1},
	{0,-1,0},
	{0,-1,1},
	{0,0,-1},
	{0,0,0},
	{0,0,1},
	{0,1,-1},
	{0,1,0},
	{0,1,1},
	{1,-1,-1},
	{1,-1,0},
	{1,-1,1},
	{1,0,-1},
	{1,0,0},
	{1,0,1},
	{1,1,-1},
	{1,1,0},
	{1,1,1},
	{0,0,0}
};

ADEVS_NAMESPACE::cellSpace::net_exec::net_exec(
devsn* parent, int width, int height, int depth, int type):
netExec(parent),
w(width),
h(height),
d(depth),
type(type)
{
	if (w == 0 || h == 0 || d == 0)
	{
		space = NULL;
		return;
	}
	space = new devs***[w];
	for (int x = 0; x < w; x++)
	{
		space[x] = new devs**[h];
		for (int y = 0; y < h; y++)
		{
			space[x][y] = new devs*[d];
			for (int z = 0; z < d; z++)
			{
				space[x][y][z] = NULL;
			}
		}
	}
}

ADEVS_NAMESPACE::cellSpace::net_exec::net_exec(const net_exec& src, devsn* parent):
netExec(src,parent),
w(src.w),
h(src.h),
d(src.d),
type(src.type)
{
	if (src.space == NULL)
	{
		space = NULL;
		return;
	}
	space = new devs***[w];
	for (int x = 0; x < w; x++)
	{
		space[x] = new devs**[h];
		for (int y = 0; y < h; y++)
		{
			space[x][y] = new devs*[d];
			for (int z = 0; z < d; z++)
			{
				if (src.space[x][y][z] == NULL)
				{
					space[x][y][z] = NULL;
				}
				else
				{
					space[x][y][z] = dynamic_cast<devs*>(src.space[x][y][z]->clone());
					space[x][y][y]->setParent(getParent());
				}
			}
		}
	}
}	

void ADEVS_NAMESPACE::cellSpace::net_exec::init()
{
	passivate();
}

void ADEVS_NAMESPACE::cellSpace::net_exec::getComponents(adevs_set<devs*>& c)
{
	for (int x = 0; x < w; x++)
	{
		for (int y = 0; y < h; y++)
		{
			for (int z = 0; z < d; z++)
			{
				if (space[x][y][z] != NULL)
				{
					c.add(space[x][y][z]);
				}
			}
		}
	}
}

void ADEVS_NAMESPACE::cellSpace::net_exec::addModel(devs* model, int x, int y, int z)
{
	space[x][y][z] = model;
}

const devs* ADEVS_NAMESPACE::cellSpace::net_exec::getModel(int x, int y, int z) const
{
	return space[x][y][z];
}

devs* ADEVS_NAMESPACE::cellSpace::net_exec::getMutableModel(int x, int y, int z)
{
	return space[x][y][z];
}

void ADEVS_NAMESPACE::cellSpace::net_exec::route(
const PortValue& pv, devs* model, adevs_bag<EventReceiver>& r)
{
	undirected_internal_coupling(pv,model,r);
	directed_internal_coupling(pv,model,r);
	external_input_coupling(pv,model,r);
	external_output_coupling(pv,model,r);
}

void ADEVS_NAMESPACE::cellSpace::net_exec::external_input_coupling(
const PortValue& pv, devs* model, adevs_bag<EventReceiver>& r)
{
	if (model == getParent())
	{
		int x, y, z;
		cell_interface::get_coord(x,y,z,w,h,pv.port);
		EventReceiver e(space[x][y][z],cell_interface::in[1][1][1]);
		r.add(e);
	}
}

void ADEVS_NAMESPACE::cellSpace::net_exec::external_output_coupling(
const PortValue& pv, devs* model, adevs_bag<EventReceiver>& r)
{
	if (model != getParent())
	{
		EventReceiver e(getParent(),getPort(model));
		r.add(e);
	}
}

void ADEVS_NAMESPACE::cellSpace::net_exec::directed_internal_coupling(
const PortValue& pv, devs* model, adevs_bag<EventReceiver>& r)
{
	if (model == getParent() || pv.port == cell_interface::out)
	{
		return;
	}
	coord_t pos;
	cell_interface::get_coord(pos.x,pos.y,pos.z,w,h,model->bsim_support.key);
	coord_t offset(cell_interface::offset_table[pv.port]);
	EventReceiver e;
	/// Get coords of neighboring cell
	int xx = pos.x + offset.x;
	int yy = pos.y + offset.y;
	int zz = pos.z + offset.z;
	/// Check that coord is valid
	if (xx >= 0 && xx < w && yy >= 0 && yy < h && zz >= 0 && zz < d)
	{
		e.model = space[xx][yy][zz];
		if (e.model != model && e.model != NULL)
		{
			e.port = cell_interface::in[1-offset.x][1-offset.y][1-offset.z];
			r.add(e);
		}
	}
	return;
}

void ADEVS_NAMESPACE::cellSpace::net_exec::undirected_internal_coupling(
const PortValue& pv, devs* model, adevs_bag<EventReceiver>& r)
{
	if (model == getParent() || pv.port != cell_interface::out)
	{
		return;
	}
	coord_t pos;
	cell_interface::get_coord(pos.x,pos.y,pos.z,w,h,model->bsim_support.key);
	EventReceiver e;
	if (type == SIX_POINT)
	{
		/// Send event left
		if (pos.x > 0)
		{
			e.model = space[pos.x-1][pos.y][pos.z];
			e.port = cell_interface::in[2][1][1];
			if (e.model != NULL) r.add(e);
		}
		/// Send event right
		if (pos.x < w-1)
		{
			e.model = space[pos.x+1][pos.y][pos.z];
			e.port = cell_interface::in[0][1][1];
			if (e.model != NULL) r.add(e);
		}
		/// Send event back
		if (pos.y > 0)
		{
			e.model = space[pos.x][pos.y-1][pos.z];
			e.port = cell_interface::in[1][2][1];
			if (e.model != NULL) r.add(e);
		}
		/// Send event forward
		if (pos.y < h-1)
		{
			e.model = space[pos.x][pos.y+1][pos.z];
			e.port = cell_interface::in[1][0][1];
			if (e.model != NULL) r.add(e);
		}
		/// Send event down
		if (pos.z > 0)
		{
			e.model = space[pos.x][pos.y][pos.z-1];
			e.port = cell_interface::in[1][1][2];
			if (e.model != NULL) r.add(e);
		}
		/// Send event up
		if (pos.z < d-1)
		{
			e.model = space[pos.x][pos.y][pos.z+1];
			e.port = cell_interface::in[1][1][0];
			if (e.model != NULL) r.add(e);
		}
	}
	else if (type == TWENTY_SIX_POINT)
	{
		for (int x = -1; x < 2; x++)
		{
			for (int y = -1; y < 2; y++)
			{
				for (int z = -1; z < 2; z++)
				{
					/// Get coords of neighboring cell
					int xx = pos.x + x;
					int yy = pos.y + y;
					int zz = pos.z + z;
					/// Check that coord is valid
					if (xx >= 0 && xx < w && yy >= 0 && yy < h && zz >= 0 && zz < d)
					{
						e.model = space[xx][yy][zz];
						if (e.model != model && e.model != NULL)
						{
							e.port = cell_interface::in[1-x][1-y][1-z];
							r.add(e);
						}
					}
				}
			}
		}
	}	
}

// Weird gcc compiler bug.  Seems to ignore whitespace when constructing
// function prototype.  Cause cellSpace::coord_t ::cellSpace... to look
// like cellSpace::coord_t::cellSpace...
#ifdef USE_ADEVS_NAMESPACE
cellSpace::coord_t ADEVS_NAMESPACE::cellSpace::net_exec::getCoordinates(
ADEVS_PORT_TYPE port) const
#else
cellSpace::coord_t cellSpace::net_exec::getCoordinates(
ADEVS_PORT_TYPE port) const
#endif
{
	coord_t coord;
	cell_interface::get_coord(coord.x,coord.y,coord.z,w,h,port);
	return coord;
}

ADEVS_PORT_TYPE ADEVS_NAMESPACE::cellSpace::net_exec::getPort(int x, int y, int z) const
{
	return cell_interface::get_key(x,y,z,w,h);
}

ADEVS_PORT_TYPE ADEVS_NAMESPACE::cellSpace::net_exec::getPort(const devs* model) const
{
	return model->bsim_support.key;
}

ADEVS_NAMESPACE::cellSpace::net_exec::~net_exec()
{
	for (int x = 0; x < w; x++)
	{
		for (int y = 0; y < h; y++)
		{
			for (int z = 0; z < d; z++)
			{
				if (space[x][y][z] != NULL)
				{
					delete space[x][y][z];
				}
			}
			delete [] space[x][y];
		}
		delete [] space[x];
	}
	delete [] space;
}

#endif
