/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2003 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include "pdevs.h"
USING_NAMESPACE_ADEVS

ADEVS_NAMESPACE::pdevs_sim::pdevs_sim(atomic* model, ADEVS_TIME_TYPE t_granule):
basic_sim(t_granule),
model(model)
{
}

void ADEVS_NAMESPACE::pdevs_sim::computeInputOutput()
{
	ADEVS_TIME_TYPE t = model->timeCurrent();
	model->asim_support.t = model->timeNext();
	model->output_func(y);
	model->asim_support.t = t;
}
	
void ADEVS_NAMESPACE::pdevs_sim::computeDeltaFunc(ADEVS_TIME_TYPE t)
{
	model->asim_support.t = t;
	if (model->sigma() <= t_granule && x.getSize() == 0)
	{
		model->delta_int();
		model->asim_support.tL = t;
		model->asim_support.tN = t + model->ta();
	}
	else if (model->sigma() <= t_granule && x.getSize() > 0)
	{
		model->delta_conf(x);
		model->asim_support.tL = t;
		model->asim_support.tN = t + model->ta();
	}
	else if (model->sigma() > t_granule && x.getSize() > 0)
	{
		model->delta_ext(model->elapsed(),x);
		model->asim_support.tL = t;
		model->asim_support.tN = t + model->ta();
	}
	x.removeAll();
}

void ADEVS_NAMESPACE::pdevs_sim::endOfCycle()
{
	model->gc_output(y);
	y.removeAll();
}

ADEVS_NAMESPACE::pdevs_sim::~pdevs_sim()
{
}
