/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2003 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include "pdevssim.h"
USING_NAMESPACE_ADEVS

ADEVS_NAMESPACE::pdevssim::pdevssim(devsn* model, int num_threads, ADEVS_TIME_TYPE t0, 
ADEVS_TIME_TYPE t_granule):
model(model),
sim(NULL)
{
	reset(num_threads,t0,t_granule);
}
	
void ADEVS_NAMESPACE::pdevssim::run(ADEVS_TIME_TYPE t_max)
{
	while (sim->timeNext() <= t_max && sim->timeNext() < ADEVS_INFINITY)
	{
		sim->computeInputOutput();
		sim->computeDeltaFunc(sim->timeNext());
		sim->endOfCycle();
	}
}

ADEVS_TIME_TYPE ADEVS_NAMESPACE::pdevssim::timeNext()
{
	return sim->timeNext();
}

void ADEVS_NAMESPACE::pdevssim::step()
{
	sim->computeInputOutput();
	sim->computeDeltaFunc(sim->timeNext());
	sim->endOfCycle();
}

void ADEVS_NAMESPACE::pdevssim::reset(int num_threads, ADEVS_TIME_TYPE t0, ADEVS_TIME_TYPE t_granule)
{
	if (sim != NULL) delete sim;
	model->initialize(t0);
	sim = new par_pdevs_coord(model,num_threads,t_granule);
}

ADEVS_NAMESPACE::pdevssim::~pdevssim()
{
	if (sim != NULL)
	{
		delete sim;
	}
}
