/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2003 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include "prefab_models.h"
using namespace std;
USING_NAMESPACE_ADEVS

ADEVS_NAMESPACE::staticDigraph::net_exec::net_exec(const staticDigraph::net_exec& src, 
devsn* parent):
netExec(src,parent)
{
	// indices
	int i, j;
	size_t k;
	// Keep track of which clones represent which components in this source model
	adevs_hashtable<const devs*,devs*> clones;
	clones.insert(src.getParent(),parent);
	// Clone all of the source component models and add them to this model
	for (i = 0; i < src.models.getSize(); i++)
	{
		devs* model = (devs*)(src.models.get(i)->clone());
		model->setParent(getParent());
		clones.insert(src.models.get(i),model);
		models.add(model);
	}
	
	vector<pair<EventReceiver,adevs_bag<EventReceiver> > > nodes(src.graph.getElements());
	// For every link in the source coupled model
	for (k = 0; k < nodes.size(); k++)
	{
		// Find the clone of the link source
		devs* source = clones[nodes[k].first.model];
		ADEVS_PORT_TYPE source_port = nodes[k].first.port;
		// For every link originating from that link source
		for (j = 0; j < nodes[k].second.getSize(); j++)
		{
			// Find the clone of the destination model
			devs* dest = clones[nodes[k].second[j].model];
			ADEVS_PORT_TYPE dest_port = nodes[k].second[j].port;
			// Couple the source clone and destination clone
			addCoupling(source,source_port,dest,dest_port);
		}
	}
}

void ADEVS_NAMESPACE::staticDigraph::net_exec::init()
{
	passivate();
}

void ADEVS_NAMESPACE::staticDigraph::net_exec::getComponents(adevs_set<devs*>& c)
{
	c.append(models);
}

void ADEVS_NAMESPACE::staticDigraph::net_exec::addModel(devs* model)
{
	if (model == this || model == getParent())
	{
		return;
	}
	else
	{
		models.add(model);
	}
}

void ADEVS_NAMESPACE::staticDigraph::net_exec::addCoupling(devs* src,
ADEVS_PORT_TYPE srcPort, devs* dst, ADEVS_PORT_TYPE dstPort)
{
	addModel(src);
	addModel(dst);
	EventReceiver p(src,srcPort);
	EventReceiver q(dst,dstPort);
	graph[p].add(q);
}

void ADEVS_NAMESPACE::staticDigraph::net_exec::route(const PortValue& pv, devs* model,
adevs_bag<EventReceiver>& r)
{
	EventReceiver src(model,pv.port);
	r.append(graph[src]);
}

ADEVS_NAMESPACE::staticDigraph::net_exec::~net_exec()
{
	for (int i = 0; i < models.getSize(); i++)
		delete models[i];
}
