/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2003 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include "xdevssim.h"
USING_NAMESPACE_ADEVS

void ADEVS_NAMESPACE::xdevssim::initialize(ADEVS_TIME_TYPE t0, ADEVS_TIME_TYPE t_granule)
{
	if (sim != NULL) delete sim;
	IO_up_to_date = false;
	input.removeAll();
	atomic* amodel = model->typeIsAtomic();
	if (amodel != NULL)
	{
		amodel->asim_support.t = t0;
		amodel->asim_support.tL = t0;
		amodel->initialize(t0);
		sim = new pdevs_sim(amodel,t_granule);
	}
	else
	{
		model->initialize(t0);
		sim = new pdevs_coord(model->typeIsNetwork(),t_granule);
	}
}

const adevs_bag<PortValue>& ADEVS_NAMESPACE::xdevssim::computeInputOutput()
{
	if (!IO_up_to_date)
	{
		sim->computeInputOutput();
		IO_up_to_date = true;
	}
	return sim->getOutput();
}
	
void ADEVS_NAMESPACE::xdevssim::deltFunc(ADEVS_TIME_TYPE t)
{
	if (!IO_up_to_date && (sim->timeNext() - t <= sim->getTimeGranule())) 
	{
		sim->computeInputOutput();
	}
	sim->inject(input);
	sim->computeDeltaFunc(t);
	sim->endOfCycle();
	input.removeAll();
	IO_up_to_date = false;
}

ADEVS_NAMESPACE::xdevssim::~xdevssim()
{
	if (sim != NULL)
	{
		delete sim;
	}
}
