#ifndef __cell_exec_h_
#define __cell_exec_h_
#include "adevs.h"
#include "cell.h"
#include <cassert>
USING_NAMESPACE_ADEVS

class cell_exec: public netExec 
{
	public:		
		const static ADEVS_PORT_TYPE birth;
		const static ADEVS_PORT_TYPE death;

		cell_exec(devsn* parent):netExec(parent){}
		void init()
		{
			cells.add(new cell());
			passivate();
		}
		void delta_int() { passivate(); }
		void delta_ext(ADEVS_TIME_TYPE e, const adevs_bag<PortValue>& x) 
		{ 
			port_iterator j(x,birth);
			for (; !j.end_of_input(); j++) 
			{
				cells.add(new cell());
			}
			port_iterator i(x,death);
			for (; !i.end_of_input(); i++) 
			{
				cells.erase((devs*)(*i));
				assert(!cells.contains((devs*)(*i)));
			}
			passivate();
		}
		void delta_conf(const adevs_bag<PortValue>& x){}
		void output_func(adevs_bag<PortValue>& y){}
		void gc_output(adevs_bag<PortValue>& g){}
		void gc_models(adevs_set<devs*>& models)
		{
			for (int i = 0; i < models.getSize(); i++)
				delete models[i];
		}
		void getComponents(adevs_set<devs*>& c)
		{
			c.append(cells);
		}
		void route(const PortValue& pv, devs* model, adevs_bag<EventReceiver>& r)
		{
			if (pv.port == cell::die)
				r.add(EventReceiver(this,death));
			else
				r.add(EventReceiver(this,birth));
		}
		~cell_exec()
		{
			for (int i = 0; i < cells.getSize(); i++)
				delete cells[i];
		}
		int getPop() const { return cells.getSize(); }

	private:	
		adevs_set<devs*> cells;
};

const ADEVS_PORT_TYPE cell_exec::death(0);
const ADEVS_PORT_TYPE cell_exec::birth(1);

class cell_space: public devsn
{
	public:
		cell_space():devsn()
		{
			nx = new cell_exec(this);
		}
		netExec* getNetExec() { return nx; }
		~cell_space() { delete nx; }
		int getPop() const { return nx->getPop(); }
	private:
		cell_exec* nx;
};

#endif
