#include "fireCell.h"
#include <fstream>
USING_NAMESPACE_ADEVS
using namespace std;

fireCell::fireCell():
atomic(),
cell_interface()
{
}

fireCell::fireCell(const fireCell& src):
atomic(src),
cell_interface(src),
phase(src.phase),
fuel(src.fuel),
heat(src.heat)
{
}

const fireCell& fireCell::operator=(const fireCell& src)
{
	atomic::operator=(src);
	cell_interface::operator=(src);
	phase = src.phase;
	fuel = src.fuel;
	heat = src.heat;
	return *this;
}

void fireCell::set_init_conditions(int fuel_, bool burning)
{
	fuel = fuel_;
	if (burning)
		phase = BURNING;
	else
		phase = UNBURNT;
	heat = 0;
}

void fireCell::init()
{
	if (phase == BURNING)
		hold(1);
	else
		passivate();
}

void fireCell::delta_int()
{
	if (phase == BURNT)
		passivate();
	else
	{
		hold(fuel-1);
		phase = BURNT;
	}
}

void fireCell::delta_ext(ADEVS_TIME_TYPE e, const adevs_bag<PortValue>& input)
{
	for (int i = 0; i < input.getSize(); i++)
	{
		if (input.get(i).value == BURNING) heat++;
		else heat--;
	}
	if (heat >= 2 && phase == UNBURNT && fuel > 0)
	{
		phase = BURNING;
		hold(1);
	}
}

void fireCell::delta_conf(const adevs_bag<PortValue>& x) 
{
	delta_int();
	delta_ext(0,x);
}

void fireCell::output_func(adevs_bag<PortValue>& y)
{
	output(out,phase,y);
}

void fireCell::gc_output(adevs_bag<PortValue>& g)
{
}

void fireCell::report(ofstream& stream) const
{
	if (phase == BURNING)
		stream << "2 ";
	else if (phase == BURNT)
		stream << "1 ";
	else if (phase == UNBURNT)
		stream << "0 ";
}

fireCell::~fireCell()
{
}

object* fireCell::clone() const
{
	return new fireCell(*this);
}
