#include "fireCell.h"
#include "configuration.h"
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <cassert>
using namespace std;
USING_NAMESPACE_ADEVS

static cellSpace* make_model(int width, int height, int** fuel, bool** fire)
{
	cellSpace* space = new cellSpace(cellSpace::TWENTY_SIX_POINT,width,height,1);
	for (int x = 0; x < width; x++)
	{
		for (int y = 0; y < height; y++)
		{
			fireCell* fire_cell = new fireCell();
			fire_cell->set_init_conditions(fuel[x][y],fire[x][y]);
			space->add(fire_cell,x,y,0);
			assert(space->getModel(x,y,0) == fire_cell);
		}
	}
	return space;
}

int main(int argc, char** argv)
{
	if (argc < 2)
	{
		std::cerr << argv[0] << " <configuration file>" << std::endl;
		std::cerr << "No input file" << std::endl;
		return 0;
	}
	configuration config(argv[1]);
	int** fuel = new int*[config.get_width()];
	bool** fire = new bool*[config.get_width()];
	for (int i = 0; i < config.get_width(); i++)
	{
		fuel[i] = new int[config.get_height()];
		fire[i] = new bool[config.get_height()];
	}
	config.get_data(0,0,config.get_width(),config.get_height(),fuel,fire);
	cellSpace* model = make_model(config.get_width(),config.get_height(),fuel,fire);
	for (int i = 0; i < config.get_width(); i++)
	{
		delete [] fuel[i];
		delete [] fire[i];
	}
	delete [] fuel;
	delete [] fire;

	cout << "Running..." << endl;
	devssim sim(model);
	sim.run((unsigned long)config.get_tend());
	ofstream log_file("soln");
	for (int y = 0; y < model->getHeight(); y++)
	{
		for (int x = 0; x < model->getWidth(); x++)
		{
			const fireCell* fire_cell=dynamic_cast<const fireCell*>(model->getModel(x,y,0));
			fire_cell->report(log_file);
		}
		log_file << endl;
	}
	log_file.close();
	delete model;
	return 0;
}
