#include "adevs_hashtable.h"
#include "adevs_exceptions.h"
#include <assert.h>
#include <iostream>
#define ARRAY_SIZE 3000
using namespace std;
USING_NAMESPACE_ADEVS

void test1()
{
	adevs_hashtable<int,int> t;
	t[1] = 1;
	t[2] = 2;
	t.remove(1);
	assert(t.getSize() == 1);
	t.remove(2);
	assert(t.getSize() == 0);
}

void test2()
{
	adevs_hashtable<int,int> t;
	t[1] = 1;
	t[2] = 2;
	t[1] = 2;
	assert(t[1] == 2);
	t.remove(1);
	assert(t.getSize() == 1);
	t.remove(2);
	assert(t.getSize() == 0);
}

void test3()
{
	adevs_hashtable<int,int> t;
	for (int i = 0; i < 200; i++)
	{
		t[i] = i;
		assert(t.contains(i));
	}
	assert(t.getSize() == 200);
	for (int i = 0; i < 200; i++)
	{
		t.remove(i);
		assert(!t.contains(i));
	}
	assert(t.getSize() == 0);
	for (int i = 0; i < 200; i++)
	{
		t[i] = i;
		assert(t.contains(i));
	}
	assert(t.getSize() == 200);
	t.clear();
	for (int i = 0; i < 200; i++)
	{
		assert(!t.contains(i));
	}
}

void test4() 
{
	int keys [ARRAY_SIZE];
	int values [ARRAY_SIZE];

	for (unsigned i = 0; i < ARRAY_SIZE; i++) 
	{
		keys[i] = i;
		values[i] = i+ARRAY_SIZE;
	}

	adevs_hashtable<int,int> t;

	for (int i = 0; i < ARRAY_SIZE; i++) 
	{
		assert (t.getSize() == i);
		t.insert(keys[i], values[i]);
		assert(t.find(keys[i]) == values[i]);
		assert(t[keys[i]] == values[i]);
		t[keys[i]] = values[i] = i;
		assert(t[keys[i]] == values[i]);
		assert(t.contains(keys[i]));
	}

	assert(t.getSize() == ARRAY_SIZE);
	adevs_hashtable<int,int> at(t);
	adevs_hashtable<int,int> bt = t;
	assert(at.getSize() == t.getSize());
	assert(bt.getSize() == t.getSize());
	t.clear();
	assert(t.getSize() == 0);
	assert(t.contains(1) == false);
	for (int i = 0; i < ARRAY_SIZE; i++) 
	{
		assert(at[keys[i]] == values[i]);
		assert(bt[keys[i]] == values[i]);
   }
	assert(at.getSize() == ARRAY_SIZE);
	assert(bt.getSize() == ARRAY_SIZE);
    
	for (int i = 0; i < ARRAY_SIZE; i++) 
	{
		int size = at.getSize();
		at.remove(keys[i]);
		assert (at.getSize() == size-1);
		bt.remove(keys[i]);
		assert(at.contains(keys[i]) == false);
		assert(bt.contains(keys[i]) == false);
	}
	
	for (int i = 0; i < ARRAY_SIZE; i++) 
	{
		assert (t.getSize() == i);
		t.insert(keys[i], values[i]);
		assert(t.find(keys[i]) == values[i]);
		assert(keys[i] == values[i]);
		t[keys[i]] = values[i] = i;
		assert(keys[i] == values[i]);
		assert(t.contains(keys[i]));
	}
	assert(t.getSize() == ARRAY_SIZE);
}

void test5()
{
	try
	{
		adevs_hashtable<int,int> table;
		for (int i = 0; i < 50000; i++)
		{
			unsigned action = (unsigned)rand()%3;
			if (action <= 1)
			{
				int item = rand();
				int key = rand();
				table.insert(key,item);
				assert(table[key] == item);
			}
			else if (action == 2 && table.getSize() == 0)
			{
				for (int k = 0; k < 10; k++)
				{
					assert(!table.contains(rand()));
				}
			}
			else if (action == 2 && table.getSize() > 0)
			{
				vector<pair<int,int> > els(table.getElements());
				int index = (size_t)rand()%els.size();
				int key = els[index].first;
				int item = els[index].second;
				assert(table.contains(key));
				assert(table[key] == item);
				table.remove(key);
				assert(!table.contains(key));
			}
		}
	}
	catch(adevs_exception& err)
	{
		cout << err << endl;
	}
}

void test6()
{
	int v = 0;
	adevs_hashtable<int,int> table;
	assert (!table.find(0,v));
	table[1] = 2;
	assert(table.find(1,v));
	assert(v == 2);
	table.remove(1);
	assert(!table.find(1,v));
}

int main()
{
	test1();
	test2();
	test3();
	test4();
	test5();
	test6();
	return 0;
}
