#include "adevs_sched.h"
#include <iostream>
#include <cassert>
USING_NAMESPACE_ADEVS

class bogus_atomic: public atomic
{
	public:
		bogus_atomic():
		atomic()
		{
		}
		void init(){}
		void delta_int(){}
		void delta_ext(ADEVS_TIME_TYPE e, const adevs_bag<PortValue>& x){}
		void delta_conf(const adevs_bag<PortValue>& x){}
		void output_func(adevs_bag<PortValue>& y){}
		void gc_output(adevs_bag<PortValue>& g){}
};
	
void testa()
{
	adevs_sched q;
	bogus_atomic m;
	q.schedule(&m,0.0);
	q.removeMinimum();
	q.schedule(&m,0.0);
	q.schedule(&m,1.0);
	assert(q.minPriority() == 1.0);
}

void test1()
{
	bogus_atomic m[10];
	adevs_sched q;
	for (int i = 0; i < 10; i++)
	{
		q.schedule(&(m[i]),(double)i);
		assert(q.getSize() == i+1);
		assert(q.minPriority() == 0.0);
		assert(q.getMinimum() == &(m[0]));
	}
	for (int i = 0; i < 10; i++)
	{
		assert(q.getSize() == 10-i);
		assert(q.minPriority() == (double)i);
		assert(q.getMinimum() == &(m[i]));
		q.removeMinimum();
	}
}

void test2()
{
	bogus_atomic m[5];
	adevs_sched q;
	q.schedule(&(m[0]),1.0);
	q.schedule(&(m[1]),10.0);
	q.schedule(&(m[2]),5.0);
	assert(q.minPriority() == 1.0);
	q.removeMinimum();
	assert(q.getSize() == 2);
	assert(q.minPriority() == 5.0);
	q.schedule(&(m[3]),3.0);
	q.schedule(&(m[4]),4.0);
	assert(q.minPriority() == 3.0);
	q.removeMinimum();
	assert(q.minPriority() == 4.0);
	q.removeMinimum();
	assert(q.minPriority() == 5.0);
	q.removeMinimum();
	assert(q.minPriority() == 10.0);
	q.removeMinimum();
	assert(q.getSize() == 0);
}

void test3()
{
	bogus_atomic m[3];
	adevs_sched q;
	q.schedule(&(m[0]),5.0);
	q.schedule(&(m[1]),10.0);
	q.schedule(&(m[2]),1.0);
	assert(q.getSize() == 3);
	q.schedule(&(m[0]),ADEVS_INFINITY);
	assert(q.getSize() == 2);
	assert(q.minPriority() == 1.0);
	q.schedule(&(m[2]),ADEVS_INFINITY);
	assert(q.getSize() == 1);
	assert(q.minPriority() == 10.0);
}

void test4()
{
	adevs_sched q;
	assert(q.minPriority() == ADEVS_INFINITY);
	assert(q.getMinimum() == NULL);
	bogus_atomic m[200];
	srand(time(NULL));
	for (int i = 0; i < 300; i++)
	{
		q.schedule(&(m[rand()%200]),(double)rand());
	}
	double tL = q.minPriority();
	while (q.getSize() > 0)
	{
		assert(q.getMinimum() != NULL);
		q.removeMinimum();
		if (q.getSize() != 0)
		{
			assert(tL <= q.minPriority());
			tL = q.minPriority();
		}
	}
	assert(q.getMinimum() == NULL);
	assert(q.minPriority() == ADEVS_INFINITY);
}

void test5()
{
	bogus_atomic m[2];
	adevs_sched q;
	q.schedule(&(m[0]),2.0);
	q.schedule(&(m[1]),3.0);
	q.schedule(&(m[1]),ADEVS_INFINITY);
	assert(q.minPriority() == 2.0);
	assert(q.getMinimum() == &(m[0]));
}

void test6()
{
	bogus_atomic m[2];
	adevs_sched q;
	q.schedule(&(m[0]),1.0);
	q.schedule(&(m[1]),1.0);
	q.schedule(&(m[0]),ADEVS_INFINITY);
	assert(q.getMinimum() == &(m[1]));
	assert(q.minPriority() == 1.0);
}

void test7()
{
	bogus_atomic m[2];
	adevs_sched q;
	q.schedule(&(m[0]),2.0);
	q.schedule(&(m[1]),3.0);
	q.schedule(&(m[0]),4.0);
	q.schedule(&(m[0]),4.0);
	assert(q.getMinimum() == &(m[1]));
	q.removeMinimum();
	assert(q.getMinimum() == &(m[0]));
	q.schedule(&(m[1]),1.0);
	assert(q.getMinimum() == &(m[1]));
}

void test8()
{
	bogus_atomic m[2000];
	adevs_sched q;
	for (int i = 0; i < 2000; i++)
	{
		q.schedule(&(m[i]),(double)i);
		assert(q.getMinimum() == &(m[i]));
		q.schedule(&(m[i]),(double)i);
		assert(q.getMinimum() == &(m[i]));
		q.schedule(&(m[i]),ADEVS_INFINITY);
		assert(q.getSize() == 0);
	}
}

int main () 
{
	testa();
	test1();
	test2();
	test3();
	test4();
	test5();
	test6();
	test7();
	test8();
	return 0;
}
