#include <iostream>
#include <adevs.h>
#include <assert.h>
USING_NAMESPACE_ADEVS

class genr: public atomic
{
	public:
		static const ADEVS_PORT_TYPE out;

		genr(ADEVS_TIME_TYPE period, int ticks, ADEVS_TIME_TYPE t_start = ADEVS_ZERO):
		atomic(),period(period),ticks(ticks),count(0),t_start(t_start) {}
		void init()
		{
			count = 0;
			hold(period);
			assert(timeCurrent() == t_start);
			assert(timeLast() == t_start);
		}
		void delta_int()
		{
			assert(timeCurrent() == timeLast()+ta());
			assert(elapsed() == ta());
			count++;
			assert(count <= ticks);
			if (count == ticks) passivate();
			else hold(period);
		}
		void delta_ext(ADEVS_TIME_TYPE e, const adevs_bag<PortValue>& x)
		{
			assert(false);
		}
		void delta_conf(const adevs_bag<PortValue>& x)
		{
			assert(false);
		}
		void output_func(adevs_bag<PortValue>& y)
		{
			output(out,new object(),y);
		}
		void gc_output(adevs_bag<PortValue>& g)
		{
			for (int i = 0; i < g.getSize(); i++) 
				delete g[i].value;
		}
		~genr(){}
		int getTickCount() { return count; }
	private:
		double period;
		int ticks;
		int count;
		ADEVS_TIME_TYPE t_start;
};

const ADEVS_PORT_TYPE genr::out(1);

void test1()
{
	genr* g = new genr(10.0,10);
	xdevssim sim(g);
	sim.initialize();
	while(sim.nextTN() < ADEVS_INFINITY)
	{
		sim.deltFunc(sim.nextTN());
	}
	assert(sim.nextTN() == ADEVS_INFINITY);
	sim.finalize();
	assert(g->getTickCount() == 10);
	delete g;
}

void test2()
{
	genr* g = new genr(-1.0,10);
	try
	{
		xdevssim sim(g);
		sim.initialize();
		assert(false);
	}
	catch(adevs_exception& err)
	{
		std::cerr<<err.getErrorMsg()<<std::endl;
	}
	delete g;
}

void test3()
{
	genr* g = new genr(1.0,10);
	xdevssim sim(g);
	sim.initialize();
	while (sim.nextTN() < ADEVS_INFINITY)
	{
		sim.computeInputOutput();
		sim.computeInputOutput();
		sim.deltFunc(sim.nextTN());
	}
	assert(sim.nextTN() == ADEVS_INFINITY);
	sim.finalize();
	assert(g->getTickCount() == 10);
	delete g;
}

void test4()
{
	double t_start = (double)time(NULL);
	genr* g = new genr(1.0,10,t_start);
	xdevssim sim(g);
	sim.initialize(t_start);
	while (sim.nextTN() < ADEVS_INFINITY)
	{
		sim.computeInputOutput();
		sim.computeInputOutput();
		sim.deltFunc(sim.nextTN());
	}
	assert(sim.nextTN() == ADEVS_INFINITY);
	sim.finalize();
	assert(g->getTickCount() == 10);
	delete g;
}


int main()
{
	test1();
	test2();
	test3();
	test4();
	return 0;
}
